@@ -3,10 +3,10 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2015 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -14,10 +14,13 @@
  * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights
  *                         reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -30,12 +33,11 @@
 
 #include <float.h>
 
-#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OMPI_BUILD_MPI_PROFILING
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_Type_create_f90_real = PMPI_Type_create_f90_real
 #endif
-
-#if OMPI_PROFILING_DEFINES
-#include "ompi/mpi/c/profile/defines.h"
+#define MPI_Type_create_f90_real PMPI_Type_create_f90_real
 #endif
 
 static const char FUNC_NAME[] = "MPI_Type_create_f90_real";
@@ -44,8 +46,7 @@ static const char FUNC_NAME[] = "MPI_Type_create_f90_real";
 int MPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype)
 {
     uint64_t key;
-
-    OPAL_CR_NOOP_PROGRESS();
+    int p_key, r_key;
 
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
@@ -63,8 +64,10 @@ int MPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype)
     /* if the user does not care about p or r set them to 0 so the
      * test associate with them will always succeed.
      */
-    if( MPI_UNDEFINED == p ) p = 0;
-    if( MPI_UNDEFINED == r ) r = 0;
+    p_key = p;
+    r_key = r;
+    if( MPI_UNDEFINED == p ) p_key = 0;
+    if( MPI_UNDEFINED == r ) r_key = 0;
 
     /**
      * With respect to the MPI standard, MPI-2.0 Sect. 10.2.5, MPI_TYPE_CREATE_F90_xxxx,
@@ -82,10 +85,10 @@ int MPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype)
 
     if( *newtype != &ompi_mpi_datatype_null.dt ) {
         ompi_datatype_t* datatype;
-        const int* a_i[2] = {&r, &p};
+        const int* a_i[2] = {&p, &r};
         int rc;
 
-        key = (((uint64_t)p) << 32) | ((uint64_t)r);
+        key = (((uint64_t)p_key) << 32) | ((uint64_t)r_key);
         if( OPAL_SUCCESS == opal_hash_table_get_value_uint64( &ompi_mpi_f90_real_hashtable,
                                                               key, (void**)newtype ) ) {
             return MPI_SUCCESS;
@@ -105,7 +108,7 @@ int MPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype)
         snprintf(datatype->name, MPI_MAX_OBJECT_NAME, "COMBINER %s",
                  (*newtype)->name);
 
-        ompi_datatype_set_args( datatype, 1, a_i, 0, NULL, 0, NULL, MPI_COMBINER_F90_REAL );
+        ompi_datatype_set_args( datatype, 2, a_i, 0, NULL, 0, NULL, MPI_COMBINER_F90_REAL );
 
         rc = opal_hash_table_set_value_uint64( &ompi_mpi_f90_real_hashtable, key, datatype );
         if (OMPI_SUCCESS != rc) {
