@@ -1,19 +1,24 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2011 The University of Tennessee and The University
+ * Copyright (c) 2004-2016 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc. All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -26,21 +31,20 @@
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/memchecker.h"
 
-#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OMPI_BUILD_MPI_PROFILING
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_Type_create_subarray = PMPI_Type_create_subarray
 #endif
-
-#if OMPI_PROFILING_DEFINES
-#include "ompi/mpi/c/profile/defines.h"
+#define MPI_Type_create_subarray PMPI_Type_create_subarray
 #endif
 
 static const char FUNC_NAME[] = "MPI_Type_create_subarray";
 
 
 int MPI_Type_create_subarray(int ndims,
-                             int size_array[],
-                             int subsize_array[],
-                             int start_array[],
+                             const int size_array[],
+                             const int subsize_array[],
+                             const int start_array[],
                              int order,
                              MPI_Datatype oldtype,
                              MPI_Datatype *newtype)
@@ -49,13 +53,13 @@ int MPI_Type_create_subarray(int ndims,
 
     MEMCHECKER(
         memchecker_datatype(oldtype);
-    );
+        );
 
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
         if( ndims < 0 ) {
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COUNT, FUNC_NAME);
-        } else if( (NULL == size_array) || (NULL == subsize_array) || (NULL == start_array) ) {
+        } else if( (ndims > 0) && ((NULL == size_array) || (NULL == subsize_array) || (NULL == start_array)) ) {
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_ARG, FUNC_NAME);
         } else if( (NULL == oldtype) || (MPI_DATATYPE_NULL == oldtype) || (NULL == newtype) ) {
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_TYPE, FUNC_NAME);
@@ -67,28 +71,20 @@ int MPI_Type_create_subarray(int ndims,
                 return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_ARG, FUNC_NAME);
             } else if( (start_array[i] < 0) || (start_array[i] > (size_array[i] - subsize_array[i])) ) {
                 return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_ARG, FUNC_NAME);
-            } 
+            }
         }
     }
 
-    OPAL_CR_ENTER_LIBRARY();
 
     rc = ompi_datatype_create_subarray( ndims, size_array, subsize_array, start_array,
                                         order, oldtype, newtype);
     if( OMPI_SUCCESS == rc ) {
-        int* a_i[5];
-
-        a_i[0] = &ndims;
-        a_i[1] = size_array;
-        a_i[2] = subsize_array;
-        a_i[3] = start_array;
-        a_i[4] = &order;
+        const int* a_i[5] = {&ndims, size_array, subsize_array, start_array, &order};
 
         ompi_datatype_set_args( *newtype, 3 * ndims + 2, a_i, 0, NULL, 1, &oldtype,
                                 MPI_COMBINER_SUBARRAY );
     }
 
-    OPAL_CR_EXIT_LIBRARY();
 
     OMPI_ERRHANDLER_RETURN(rc, MPI_COMM_WORLD, rc, FUNC_NAME);
 }
