@@ -6,43 +6,40 @@
  * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2009 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2007-2013 Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ * Copyright (c) 2006-2016 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2015 Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * Copyright (c) 2013      NVIDIA Corporation.  All rights reserved.
- * Copyright (c) 2013      Intel, Inc. All rights reserved
+ * Copyright (c) 2013-2014 Intel, Inc. All rights reserved
+ * Copyright (c) 2015      Mellanox Technologies, Inc.
+ *                         All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
 #include "ompi_config.h"
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
-#ifdef HAVE_TIME_H
 #include <time.h>
-#endif  /* HAVE_TIME_H */
 
-#include "opal/mca/base/base.h"
 #include "ompi/constants.h"
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/runtime/mpiruntime.h"
 #include "ompi/runtime/params.h"
 #include "ompi/mca/rte/rte.h"
 
-#include "opal/mca/base/mca_base_param.h"
 #include "opal/util/argv.h"
 #include "opal/util/output.h"
 #include "opal/util/show_help.h"
-
+#include "opal/runtime/opal.h"
+#include "opal/runtime/opal_params.h"
 /*
  * Global variables
  *
@@ -58,23 +55,20 @@ int ompi_debug_show_mpi_alloc_mem_leaks = 0;
 bool ompi_debug_no_free_handles = false;
 bool ompi_mpi_show_mca_params = false;
 char *ompi_mpi_show_mca_params_file = NULL;
-bool ompi_mpi_abort_print_stack = false;
-int ompi_mpi_abort_delay = 0;
 bool ompi_mpi_keep_fqdn_hostnames = false;
-int ompi_mpi_leave_pinned = -1;
-bool ompi_mpi_leave_pinned_pipeline = false;
 bool ompi_have_sparse_group_storage = OPAL_INT_TO_BOOL(OMPI_GROUP_SPARSE);
 bool ompi_use_sparse_group_storage = OPAL_INT_TO_BOOL(OMPI_GROUP_SPARSE);
-bool ompi_mpi_built_with_cuda_support = OPAL_INT_TO_BOOL(OPAL_CUDA_SUPPORT);
-bool ompi_mpi_cuda_support;
 
-uint32_t ompi_hostname_cutoff = UINT32_MAX;
 bool ompi_mpi_yield_when_idle = true;
 int ompi_mpi_event_tick_rate = -1;
 char *ompi_mpi_show_mca_params_string = NULL;
 bool ompi_mpi_have_sparse_group_storage = !!(OMPI_GROUP_SPARSE);
 bool ompi_mpi_preconnect_mpi = false;
 
+#define OMPI_ADD_PROCS_CUTOFF_DEFAULT 0
+uint32_t ompi_add_procs_cutoff = OMPI_ADD_PROCS_CUTOFF_DEFAULT;
+bool ompi_mpi_dynamics_enabled = true;
+
 static bool show_default_mca_params = false;
 static bool show_file_mca_params = false;
 static bool show_enviro_mca_params = false;
@@ -95,12 +89,12 @@ int ompi_mpi_register_params(void)
                                  MCA_BASE_VAR_SCOPE_READONLY,
                                  &ompi_mpi_param_check);
     if (ompi_mpi_param_check && !MPI_PARAM_CHECK) {
-        opal_show_help("help-mpi-runtime.txt", 
+        opal_show_help("help-mpi-runtime.txt",
                        "mpi-param-check-enabled-but-compiled-out",
                        true);
         ompi_mpi_param_check = false;
     }
-    
+
     /*
      * opal_progress: decide whether to yield and the event library
      * tick rate
@@ -131,7 +125,7 @@ int ompi_mpi_register_params(void)
                                  OPAL_INFO_LVL_9,
                                  MCA_BASE_VAR_SCOPE_READONLY,
                                  &ompi_debug_show_handle_leaks);
-    
+
     /* Whether or not to free MPI handles.  Useless without run-time
        param checking, so implicitly set that to true if we don't want
        to free the handles. */
@@ -173,7 +167,7 @@ int ompi_mpi_register_params(void)
     if (NULL != ompi_mpi_show_mca_params_string) {
         char **args;
         int i;
-        
+
         ompi_mpi_show_mca_params = true;
         args = opal_argv_split(ompi_mpi_show_mca_params_string, ',');
         if (NULL == args) {
@@ -211,36 +205,9 @@ int ompi_mpi_register_params(void)
                                  OPAL_INFO_LVL_9,
                                  MCA_BASE_VAR_SCOPE_READONLY,
                                  &ompi_mpi_show_mca_params_file);
-    
+
     /* User-level process pinning controls */
 
-    /* MPI_ABORT controls */
-    ompi_mpi_abort_delay = 0;
-    (void) mca_base_var_register("ompi", "mpi", NULL, "abort_delay",
-                                "If nonzero, print out an identifying message when MPI_ABORT is invoked (hostname, PID of the process that called MPI_ABORT) and delay for that many seconds before exiting (a negative delay value means to never abort).  This allows attaching of a debugger before quitting the job.",
-                                 MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
-                                 OPAL_INFO_LVL_9,
-                                 MCA_BASE_VAR_SCOPE_READONLY,
-                                 &ompi_mpi_abort_delay);
-
-    ompi_mpi_abort_print_stack = false;
-    (void) mca_base_var_register("ompi", "mpi", NULL, "abort_print_stack",
-                                 "If nonzero, print out a stack trace when MPI_ABORT is invoked",
-                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0,
-                                /* If we do not have stack trace
-                                   capability, make this a constant
-                                   MCA variable */
-#if OPAL_WANT_PRETTY_PRINT_STACKTRACE && defined(HAVE_BACKTRACE)
-                                 0,
-                                 OPAL_INFO_LVL_9,
-                                 MCA_BASE_VAR_SCOPE_READONLY,
-#else
-                                 MCA_BASE_VAR_FLAG_DEFAULT_ONLY,
-                                 OPAL_INFO_LVL_9,
-                                 MCA_BASE_VAR_SCOPE_CONSTANT,
-#endif
-                                 &ompi_mpi_abort_print_stack);
-
     ompi_mpi_preconnect_mpi = false;
     value = mca_base_var_register("ompi", "mpi", NULL, "preconnect_mpi",
                                   "Whether to force MPI processes to fully "
@@ -257,38 +224,6 @@ int ompi_mpi_register_params(void)
     mca_base_var_register_synonym(value, "ompi", "mpi", NULL, "preconnect_all",
                                   MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
-    /* Leave pinned parameter */
-    ompi_mpi_leave_pinned = -1;
-    (void) mca_base_var_register("ompi", "mpi", NULL, "leave_pinned",
-                                 "Whether to use the \"leave pinned\" protocol or not.  Enabling this setting can help bandwidth performance when repeatedly sending and receiving large messages with the same buffers over RDMA-based networks (0 = do not use \"leave pinned\" protocol, 1 = use \"leave pinned\" protocol, -1 = allow network to choose at runtime).",
-                                 MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
-                                 OPAL_INFO_LVL_9,
-                                 MCA_BASE_VAR_SCOPE_READONLY,
-                                 &ompi_mpi_leave_pinned);
-
-    ompi_mpi_leave_pinned_pipeline = false;
-    (void) mca_base_var_register("ompi", "mpi", NULL, "leave_pinned_pipeline",
-                                "Whether to use the \"leave pinned pipeline\" protocol or not.",
-                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                 OPAL_INFO_LVL_9,
-                                 MCA_BASE_VAR_SCOPE_READONLY,
-                                 &ompi_mpi_leave_pinned_pipeline);
-    
-    if (ompi_mpi_leave_pinned > 0 && ompi_mpi_leave_pinned_pipeline) {
-        ompi_mpi_leave_pinned_pipeline = 0;
-        opal_show_help("help-mpi-runtime.txt", 
-                       "mpi-params:leave-pinned-and-pipeline-selected",
-                       true);
-    }
-
-    ompi_warn_on_fork = true;
-    (void) mca_base_var_register("ompi", "mpi", NULL, "warn_on_fork",
-                                 "If nonzero, issue a warning if program forks under conditions that could cause system errors",
-                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                 OPAL_INFO_LVL_9,
-                                 MCA_BASE_VAR_SCOPE_READONLY,
-                                 &ompi_warn_on_fork);
-    
     /* Sparse group storage support */
     (void) mca_base_var_register("ompi", "mpi", NULL, "have_sparse_group_storage",
                                  "Whether this Open MPI installation supports storing of data in MPI groups in \"sparse\" formats (good for extremely large process count MPI jobs that create many communicators/groups)",
@@ -307,44 +242,49 @@ int ompi_mpi_register_params(void)
                                  ompi_mpi_have_sparse_group_storage ? MCA_BASE_VAR_SCOPE_READONLY : MCA_BASE_VAR_SCOPE_CONSTANT,
                                  &ompi_use_sparse_group_storage);
     if (ompi_use_sparse_group_storage && !ompi_mpi_have_sparse_group_storage) {
-        opal_show_help("help-mpi-runtime.txt", 
+        opal_show_help("help-mpi-runtime.txt",
                        "sparse groups enabled but compiled out",
                        true);
         ompi_use_sparse_group_storage = false;
     }
 
-    (void) mca_base_var_register("ompi", "mpi", NULL, "built_with_cuda_support",
-                                 "Whether CUDA GPU buffer support is built into library or not",
-                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0,
-                                 MCA_BASE_VAR_FLAG_DEFAULT_ONLY,
-                                 OPAL_INFO_LVL_4,
-                                 MCA_BASE_VAR_SCOPE_CONSTANT,
-                                 &ompi_mpi_built_with_cuda_support);
+    value = mca_base_var_find ("opal", "opal", NULL, "cuda_support");
+    if (0 <= value) {
+        mca_base_var_register_synonym(value, "ompi", "mpi", NULL, "cuda_support",
+                                      MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
+    }
 
-	/* Current default is to enable CUDA support if it is built into library */
-	ompi_mpi_cuda_support = ompi_mpi_built_with_cuda_support;
+    value = mca_base_var_find ("opal", "opal", NULL, "built_with_cuda_support");
+    if (0 <= value) {
+        mca_base_var_register_synonym(value, "ompi", "mpi", NULL, "built_with_cuda_support", 0);
+    }
 
-    (void) mca_base_var_register("ompi", "mpi", NULL, "cuda_support",
-                                 "Whether CUDA GPU buffer support is enabled or not",
-                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                 OPAL_INFO_LVL_4,
-                                 MCA_BASE_VAR_SCOPE_READONLY,
-                                 &ompi_mpi_cuda_support);
-    if (ompi_mpi_cuda_support && !ompi_mpi_built_with_cuda_support) {
+    if (opal_cuda_support && !opal_built_with_cuda_support) {
         opal_show_help("help-mpi-runtime.txt", "no cuda support",
                        true);
         ompi_rte_abort(1, NULL);
     }
 
-    /* cutoff for retrieving hostnames */
-    ompi_hostname_cutoff = UINT32_MAX;
-    (void) mca_base_var_register ("ompi", "ompi", NULL, "hostname_cutoff",
-                                  "If the number of processes in the application exceeds the provided value,"
-                                  "hostnames for remote processes will not be retrieved by applications [default: UINT32_MAX]",
-                                  MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &ompi_hostname_cutoff);
+    ompi_add_procs_cutoff = OMPI_ADD_PROCS_CUTOFF_DEFAULT;
+    (void) mca_base_var_register ("ompi", "mpi", NULL, "add_procs_cutoff",
+                                  "Maximum world size for pre-allocating resources for all "
+                                  "remote processes. Increasing this limit may improve "
+                                  "communication performance at the cost of memory usage",
+                                  MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL,
+                                  0, 0, OPAL_INFO_LVL_3, MCA_BASE_VAR_SCOPE_LOCAL,
+                                  &ompi_add_procs_cutoff);
 
+    value = mca_base_var_find ("opal", "opal", NULL, "abort_delay");
+    if (0 <= value) {
+        (void) mca_base_var_register_synonym(value, "ompi", "mpi", NULL, "abort_delay",
+                                      MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
+    }
+
+    value = mca_base_var_find ("opal", "opal", NULL, "abort_print_stack");
+    if (0 <= value) {
+        (void) mca_base_var_register_synonym(value, "ompi", "mpi", NULL, "abort_print_stack",
+                                      MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
+    }
 
     return OMPI_SUCCESS;
 }
@@ -359,9 +299,9 @@ int ompi_show_all_mca_params(int32_t rank, int requested, char *nodename) {
     if (rank != 0) {
         return OMPI_SUCCESS;
     }
-    
+
     timestamp = time(NULL);
-    
+
     /* Open the file if one is specified */
     if (0 != strlen(ompi_mpi_show_mca_params_file)) {
         if ( NULL == (fp = fopen(ompi_mpi_show_mca_params_file, "w")) ) {
@@ -385,37 +325,37 @@ int ompi_show_all_mca_params(int32_t rank, int requested, char *nodename) {
         if (MCA_BASE_VAR_FLAG_INTERNAL & var->mbv_flags) {
             continue;
         }
-        
+
         /* is this a default value and we are not displaying
          * defaults, ignore this one
          */
         if (MCA_BASE_VAR_SOURCE_DEFAULT == var->mbv_source && !show_default_mca_params) {
             continue;
         }
-        
+
         /* is this a file value and we are not displaying files,
          * ignore it
          */
         if ((MCA_BASE_VAR_SOURCE_FILE == var->mbv_source ||
-             MCA_BASE_VAR_SOURCE_OVERRIDE == var->mbv_source) && 
+             MCA_BASE_VAR_SOURCE_OVERRIDE == var->mbv_source) &&
             !show_file_mca_params) {
             continue;
         }
-        
+
         /* is this an enviro value and we are not displaying enviros,
          * ignore it
          */
         if (MCA_BASE_VAR_SOURCE_ENV == var->mbv_source && !show_enviro_mca_params) {
             continue;
         }
-        
+
         /* is this an API value and we are not displaying APIs,
          * ignore it
          */
         if (MCA_BASE_VAR_SOURCE_OVERRIDE == var->mbv_source && !show_override_mca_params) {
             continue;
         }
-        
+
         ret = mca_base_var_dump (i, &var_dump, MCA_BASE_VAR_DUMP_SIMPLE);
         if (OPAL_SUCCESS != ret) {
             continue;
@@ -430,11 +370,11 @@ int ompi_show_all_mca_params(int32_t rank, int requested, char *nodename) {
         free (var_dump[0]);
         free (var_dump);
     }
-    
+
     /* Close file, cleanup allocated memory*/
     if (0 != strlen(ompi_mpi_show_mca_params_file)) {
         fclose(fp);
     }
-    
+
     return OMPI_SUCCESS;
 }
