@@ -1,23 +1,23 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
-/* 
+/*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2013-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -41,6 +41,16 @@ BEGIN_C_DECLS
 #define OMPI_WIN_FREED        0x00000001
 #define OMPI_WIN_INVALID      0x00000002
 #define OMPI_WIN_NO_LOCKS     0x00000004
+#define OMPI_WIN_SAME_DISP    0x00000008
+#define OMPI_WIN_SAME_SIZE    0x00000010
+
+enum ompi_win_accumulate_ops_t {
+    OMPI_WIN_ACCUMULATE_OPS_SAME_OP_NO_OP,
+    OMPI_WIN_ACCUMULATE_OPS_SAME_OP,
+};
+typedef enum ompi_win_accumulate_ops_t ompi_win_accumulate_ops_t;
+
+OMPI_DECLSPEC extern mca_base_var_enum_t *ompi_win_accumulate_ops;
 
 OMPI_DECLSPEC extern opal_pointer_array_t ompi_mpi_windows;
 
@@ -57,6 +67,12 @@ struct ompi_win_t {
     /* Information about the state of the window.  */
     uint16_t w_flags;
 
+    /** Window flavor */
+    uint16_t w_flavor;
+
+    /** Accumulate ops */
+    ompi_win_accumulate_ops_t w_acc_ops;
+
     /* Attributes */
     opal_hash_table_t *w_keyhash;
 
@@ -94,7 +110,7 @@ OMPI_DECLSPEC extern ompi_predefined_win_t *ompi_mpi_win_null_addr;
 int ompi_win_init(void);
 int ompi_win_finalize(void);
 
-int ompi_win_create(void *base, size_t size, int disp_unit, 
+int ompi_win_create(void *base, size_t size, int disp_unit,
                     ompi_communicator_t *comm, ompi_info_t *info,
                     ompi_win_t **newwin);
 int ompi_win_allocate(size_t size, int disp_unit, ompi_info_t *info,
@@ -114,7 +130,7 @@ OMPI_DECLSPEC int ompi_win_group(ompi_win_t *win, ompi_group_t **group);
    to the defintion of an "invalid" communicator.  See a big comment
    in ompi/communicator/communicator.h about this. */
 static inline int ompi_win_invalid(ompi_win_t *win) {
-    if (NULL == win || 
+    if (NULL == win ||
         MPI_WIN_NULL == win ||
         (OMPI_WIN_INVALID & win->w_flags) ||
         (OMPI_WIN_FREED & win->w_flags)) {
@@ -125,7 +141,7 @@ static inline int ompi_win_invalid(ompi_win_t *win) {
 }
 
 static inline int ompi_win_peer_invalid(ompi_win_t *win, int peer) {
-    if (win->w_group->grp_proc_count <= peer) return true;
+    if (win->w_group->grp_proc_count <= peer || peer < 0) return true;
     return false;
 }
 
