@@ -9,6 +9,9 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2015 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -20,6 +23,7 @@
 
 #include "opal/util/output.h"
 #include "opal/dss/dss_internal.h"
+#include "opal/util/error.h"
 
 int opal_dss_copy(void **dest, void *src, opal_data_type_t type)
 {
@@ -101,6 +105,14 @@ int opal_dss_std_copy(void **dest, void *src, opal_data_type_t type)
         datasize = sizeof(struct timeval);
         break;
 
+    case OPAL_TIME:
+        datasize = sizeof(time_t);
+        break;
+
+    case OPAL_NAME:
+        datasize = sizeof(opal_process_name_t);
+        break;
+
     default:
         return OPAL_ERR_UNKNOWN_DATA_TYPE;
     }
@@ -125,7 +137,7 @@ int opal_dss_copy_null(char **dest, char *src, opal_data_type_t type)
 {
     char *val;
 
-    *dest = (char*)malloc(sizeof(char*));
+    *dest = (char*)malloc(sizeof(char));
     if (NULL == *dest) {
         return OPAL_ERR_OUT_OF_RESOURCE;
     }
@@ -189,14 +201,14 @@ int opal_dss_copy_pstat(opal_pstats_t **dest, opal_pstats_t *src,
                         opal_data_type_t type)
 {
     opal_pstats_t *p;
-    
+
     /* create the new object */
     *dest = OBJ_NEW(opal_pstats_t);
     if (NULL == *dest) {
         return OPAL_ERR_OUT_OF_RESOURCE;
     }
     p = *dest;
-    
+
     /* copy the individual fields */
     memcpy(p->node, src->node, sizeof(src->node));
     p->rank = src->rank;
@@ -211,7 +223,7 @@ int opal_dss_copy_pstat(opal_pstats_t **dest, opal_pstats_t *src,
     p->peak_vsize = src->peak_vsize;
     p->processor = src->processor;
     p->sample_time.tv_sec = src->sample_time.tv_sec;
-    p->sample_time.tv_usec = src->sample_time.tv_usec;    
+    p->sample_time.tv_usec = src->sample_time.tv_usec;
     return OPAL_SUCCESS;
 }
 
@@ -220,14 +232,14 @@ int opal_dss_copy_node_stat(opal_node_stats_t **dest, opal_node_stats_t *src,
                             opal_data_type_t type)
 {
     opal_node_stats_t *p;
-    
+
     /* create the new object */
     *dest = OBJ_NEW(opal_node_stats_t);
     if (NULL == *dest) {
         return OPAL_ERR_OUT_OF_RESOURCE;
     }
     p = *dest;
-    
+
     /* copy the individual fields */
     p->la = src->la;
     p->la5 = src->la5;
@@ -235,7 +247,7 @@ int opal_dss_copy_node_stat(opal_node_stats_t **dest, opal_node_stats_t *src,
     p->total_mem = src->total_mem;
     p->free_mem = src->free_mem;
     p->sample_time.tv_sec = src->sample_time.tv_sec;
-    p->sample_time.tv_usec = src->sample_time.tv_usec;    
+    p->sample_time.tv_usec = src->sample_time.tv_usec;
     return OPAL_SUCCESS;
 }
 
@@ -244,19 +256,18 @@ int opal_dss_copy_value(opal_value_t **dest, opal_value_t *src,
                         opal_data_type_t type)
 {
     opal_value_t *p;
-    
+
     /* create the new object */
     *dest = OBJ_NEW(opal_value_t);
     if (NULL == *dest) {
         return OPAL_ERR_OUT_OF_RESOURCE;
     }
     p = *dest;
-    
+
     /* copy the type and key */
     if (NULL != src->key) {
         p->key = strdup(src->key);
     }
-    p->scope = src->scope;
     p->type = src->type;
 
     /* copy the right field */
@@ -275,34 +286,42 @@ int opal_dss_copy_value(opal_value_t **dest, opal_value_t *src,
         p->data.pid = src->data.pid;
         break;
     case OPAL_INT:
-        p->data.integer = src->data.integer;
+        /* to avoid alignment issues */
+        memcpy(&p->data.integer, &src->data.integer, sizeof(int));
         break;
     case OPAL_INT8:
         p->data.int8 = src->data.int8;
         break;
     case OPAL_INT16:
-        p->data.int16 = src->data.int16;
+        /* to avoid alignment issues */
+        memcpy(&p->data.int16, &src->data.int16, 2);
         break;
     case OPAL_INT32:
-        p->data.int32 = src->data.int32;
+        /* to avoid alignment issues */
+        memcpy(&p->data.int32, &src->data.int32, 4);
         break;
     case OPAL_INT64:
-        p->data.int64 = src->data.int64;
+        /* to avoid alignment issues */
+        memcpy(&p->data.int64, &src->data.int64, 8);
         break;
     case OPAL_UINT:
-        p->data.uint = src->data.uint;
+        /* to avoid alignment issues */
+        memcpy(&p->data.uint, &src->data.uint, sizeof(unsigned int));
         break;
     case OPAL_UINT8:
         p->data.uint8 = src->data.uint8;
         break;
     case OPAL_UINT16:
-        p->data.uint16 = src->data.uint16;
+        /* to avoid alignment issues */
+        memcpy(&p->data.uint16, &src->data.uint16, 2);
         break;
     case OPAL_UINT32:
-        p->data.uint32 = src->data.uint32;
+        /* to avoid alignment issues */
+        memcpy(&p->data.uint32, &src->data.uint32, 4);
         break;
     case OPAL_UINT64:
-        p->data.uint64 = src->data.uint64;
+        /* to avoid alignment issues */
+        memcpy(&p->data.uint64, &src->data.uint64, 8);
         break;
     case OPAL_BYTE_OBJECT:
         if (NULL != src->data.bo.bytes && 0 < src->data.bo.size) {
@@ -314,6 +333,9 @@ int opal_dss_copy_value(opal_value_t **dest, opal_value_t *src,
             p->data.bo.size = 0;
         }
         break;
+    case OPAL_NAME:
+        memcpy(&p->data.name, &src->data.name, sizeof(opal_process_name_t));
+        break;
     default:
         opal_output(0, "COPY-OPAL-VALUE: UNSUPPORTED TYPE %d", (int)src->type);
         return OPAL_ERROR;
@@ -325,6 +347,63 @@ int opal_dss_copy_value(opal_value_t **dest, opal_value_t *src,
 int opal_dss_copy_buffer_contents(opal_buffer_t **dest, opal_buffer_t *src,
                                   opal_data_type_t type)
 {
+    *dest = OBJ_NEW(opal_buffer_t);
+    opal_dss.copy_payload(*dest, src);
     return OPAL_SUCCESS;
 }
 
+/* PROCESS NAME */
+int opal_dss_copy_name(opal_process_name_t **dest, opal_process_name_t *src, opal_data_type_t type)
+{
+    opal_process_name_t *val;
+
+    val = (opal_process_name_t*)malloc(sizeof(opal_process_name_t));
+    if (NULL == val) {
+        OPAL_ERROR_LOG(OPAL_ERR_OUT_OF_RESOURCE);
+        return OPAL_ERR_OUT_OF_RESOURCE;
+    }
+
+    val->jobid = src->jobid;
+    val->vpid = src->vpid;
+
+    *dest = val;
+    return OPAL_SUCCESS;
+}
+
+/*
+ * JOBID
+ */
+int opal_dss_copy_jobid(opal_jobid_t **dest, opal_jobid_t *src, opal_data_type_t type)
+{
+    opal_jobid_t *val;
+
+    val = (opal_jobid_t*)malloc(sizeof(opal_jobid_t));
+    if (NULL == val) {
+        OPAL_ERROR_LOG(OPAL_ERR_OUT_OF_RESOURCE);
+        return OPAL_ERR_OUT_OF_RESOURCE;
+    }
+
+    *val = *src;
+    *dest = val;
+
+    return OPAL_SUCCESS;
+}
+
+/*
+ * VPID
+ */
+int opal_dss_copy_vpid(opal_vpid_t **dest, opal_vpid_t *src, opal_data_type_t type)
+{
+    opal_vpid_t *val;
+
+    val = (opal_vpid_t*)malloc(sizeof(opal_vpid_t));
+    if (NULL == val) {
+        OPAL_ERROR_LOG(OPAL_ERR_OUT_OF_RESOURCE);
+        return OPAL_ERR_OUT_OF_RESOURCE;
+    }
+
+    *val = *src;
+    *dest = val;
+
+    return OPAL_SUCCESS;
+}
