@@ -11,7 +11,11 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007-2011 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2012-2013 Los Alamos National Security, Inc. All rights reserved.
+ * Copyright (c) 2012-2013 Los Alamos National Security, Inc. All rights
+ *                         reserved.
+ * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2016 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,7 +33,7 @@
 
 #include "opal_config.h"
 
-#ifdef OPAL_HAVE_SYS_TIME_H
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h> /* for struct timeval */
 #endif
 
@@ -37,6 +41,14 @@
 #include "opal/class/opal_pointer_array.h"
 #include "opal/class/opal_list.h"
 
+typedef uint32_t opal_jobid_t;
+typedef uint32_t opal_vpid_t;
+typedef struct {
+    opal_jobid_t jobid;
+    opal_vpid_t vpid;
+} opal_process_name_t;
+#define OPAL_SIZEOF_PROCESS_NAME_T 8
+
 BEGIN_C_DECLS
 
 typedef uint8_t opal_data_type_t;  /** data type indicators */
@@ -69,53 +81,152 @@ typedef struct {
 #define    OPAL_UINT16              (opal_data_type_t)   13 /**< a 16-bit unsigned integer */
 #define    OPAL_UINT32              (opal_data_type_t)   14 /**< a 32-bit unsigned integer */
 #define    OPAL_UINT64              (opal_data_type_t)   15 /**< a 64-bit unsigned integer */
-    /* simple floating point type */
+    /* floating point types */
 #define    OPAL_FLOAT               (opal_data_type_t)   16
+#define    OPAL_DOUBLE              (opal_data_type_t)   17
     /* system types */
-#define OPAL_TIMEVAL                (opal_data_type_t)   17
+#define    OPAL_TIMEVAL             (opal_data_type_t)   18
+#define    OPAL_TIME                (opal_data_type_t)   19
     /* OPAL types */
-#define    OPAL_BYTE_OBJECT         (opal_data_type_t)   18 /**< byte object structure */
-#define    OPAL_DATA_TYPE           (opal_data_type_t)   19 /**< data type */
-#define    OPAL_NULL                (opal_data_type_t)   20 /**< don't interpret data type */
-#define    OPAL_PSTAT               (opal_data_type_t)   21 /**< process statistics */
-#define    OPAL_NODE_STAT           (opal_data_type_t)   22 /**< node statistics */
-#define    OPAL_HWLOC_TOPO          (opal_data_type_t)   23 /**< hwloc topology */
-#define    OPAL_VALUE               (opal_data_type_t)   24 /**< opal value structure */
-#define    OPAL_BUFFER              (opal_data_type_t)   25 /**< pack the remaining contents of a buffer as an object */
-
-#define    OPAL_DSS_ID_DYNAMIC      (opal_data_type_t)   30
+#define    OPAL_BYTE_OBJECT         (opal_data_type_t)   20 /**< byte object structure */
+#define    OPAL_DATA_TYPE           (opal_data_type_t)   21 /**< data type */
+#define    OPAL_NULL                (opal_data_type_t)   22 /**< don't interpret data type */
+#define    OPAL_PSTAT               (opal_data_type_t)   23 /**< process statistics */
+#define    OPAL_NODE_STAT           (opal_data_type_t)   24 /**< node statistics */
+#define    OPAL_HWLOC_TOPO          (opal_data_type_t)   25 /**< hwloc topology */
+#define    OPAL_VALUE               (opal_data_type_t)   26 /**< opal value structure */
+#define    OPAL_BUFFER              (opal_data_type_t)   27 /**< pack the remaining contents of a buffer as an object */
+#define    OPAL_PTR                 (opal_data_type_t)   28 /**< pointer to void* */
+    /* OPAL Array types */
+#define    OPAL_FLOAT_ARRAY         (opal_data_type_t)   31
+#define    OPAL_DOUBLE_ARRAY        (opal_data_type_t)   32
+#define    OPAL_STRING_ARRAY        (opal_data_type_t)   33
+#define    OPAL_BOOL_ARRAY          (opal_data_type_t)   34
+#define    OPAL_SIZE_ARRAY          (opal_data_type_t)   35
+#define    OPAL_BYTE_ARRAY          (opal_data_type_t)   36
+#define    OPAL_INT_ARRAY           (opal_data_type_t)   37
+#define    OPAL_INT8_ARRAY          (opal_data_type_t)   38
+#define    OPAL_INT16_ARRAY         (opal_data_type_t)   39
+#define    OPAL_INT32_ARRAY         (opal_data_type_t)   40
+#define    OPAL_INT64_ARRAY         (opal_data_type_t)   41
+#define    OPAL_UINT_ARRAY          (opal_data_type_t)   42
+#define    OPAL_UINT8_ARRAY         (opal_data_type_t)   43
+#define    OPAL_UINT16_ARRAY        (opal_data_type_t)   44
+#define    OPAL_UINT32_ARRAY        (opal_data_type_t)   45
+#define    OPAL_UINT64_ARRAY        (opal_data_type_t)   46
+#define    OPAL_BYTE_OBJECT_ARRAY   (opal_data_type_t)   47
+#define    OPAL_PID_ARRAY           (opal_data_type_t)   48
+#define    OPAL_TIMEVAL_ARRAY       (opal_data_type_t)   49
+#define    OPAL_NAME                (opal_data_type_t)   50
+#define    OPAL_JOBID               (opal_data_type_t)   51
+#define    OPAL_VPID                (opal_data_type_t)   52
+    /* OPAL Dynamic */
+#define    OPAL_DSS_ID_DYNAMIC      (opal_data_type_t)  100
 
 /* define the results values for comparisons so we can change them in only one place */
 #define OPAL_VALUE1_GREATER  +1
 #define OPAL_VALUE2_GREATER  -1
 #define OPAL_EQUAL            0
 
-/* define a flag to indicate the scope of data being
- * stored in the database. The following options are supported:
- *
- * PEER:     data to be shared with our peers
- * NON_PEER: data to be shared only with non-peer
- *           processes (i.e., processes from other jobs)
- * GLOBAL:   data to be shared with all processes
- * INTERNAL: data is to be internally stored in this app
- * ALL:      any of the above
- */
-typedef uint8_t opal_scope_t;
-#define OPAL_SCOPE_UNDEF    0x00
-#define OPAL_SCOPE_PEER     0x01
-#define OPAL_SCOPE_NON_PEER 0x02
-#define OPAL_SCOPE_GLOBAL   0x03
-#define OPAL_SCOPE_INTERNAL 0x80
-#define OPAL_SCOPE_ALL      0xff
-#define OPAL_DATA_SCOPE_T OPAL_UINT8
+/* List types for opal_value_t, needs number of elements and a pointer */
+/* float array object */
+typedef struct {
+    int32_t size;
+    float *data;
+} opal_float_array_t;
+/* double array object */
+typedef struct {
+    int32_t size;
+    double *data;
+} opal_double_array_t;
+/* string array object */
+typedef struct {
+    int32_t size;
+    char **data;
+} opal_string_array_t;
+/* bool array object */
+typedef struct {
+    int32_t size;
+    bool *data;
+} opal_bool_array_t;
+/* size array object */
+typedef struct {
+    int32_t size;
+    size_t *data;
+} opal_size_array_t;
+/* opal byte object array object */
+typedef struct {
+    int32_t size;
+    opal_byte_object_t *data;
+} opal_byte_object_array_t;
+/* int array object */
+typedef struct {
+    int32_t size;
+    int *data;
+} opal_int_array_t;
+/* int8 array object */
+typedef struct {
+    int32_t size;
+    int8_t *data;
+} opal_int8_array_t;
+/* int16 array object */
+typedef struct {
+    int32_t size;
+    int16_t *data;
+} opal_int16_array_t;
+/* int32 array object */
+typedef struct {
+    int32_t size;
+    int32_t *data;
+} opal_int32_array_t;
+/* int64 array object */
+typedef struct {
+    int32_t size;
+    int64_t *data;
+} opal_int64_array_t;
+/* uint array object */
+typedef struct {
+    int32_t size;
+    unsigned int *data;
+} opal_uint_array_t;
+/* uint8 array object */
+typedef struct {
+    int32_t size;
+    uint8_t *data;
+} opal_uint8_array_t;
+/* uint16 array object */
+typedef struct {
+    int32_t size;
+    uint16_t *data;
+} opal_uint16_array_t;
+/* uint32 array object */
+typedef struct {
+    int32_t size;
+    uint32_t *data;
+} opal_uint32_array_t;
+/* uint64 array object */
+typedef struct {
+    int32_t size;
+    uint64_t *data;
+} opal_uint64_array_t;
+/* pid array object */
+typedef struct {
+    int32_t size;
+    pid_t *data;
+} opal_pid_array_t;
+/* timeval array object */
+typedef struct {
+    int32_t size;
+    struct timeval *data;
+} opal_timeval_array_t;
 
 /* Data value object */
 typedef struct {
     opal_list_item_t super;             /* required for this to be on lists */
     char *key;                          /* key string */
-    opal_scope_t scope;
     opal_data_type_t type;              /* the type of value stored */
     union {
+        bool flag;
         uint8_t byte;
         char *string;
         size_t size;
@@ -132,7 +243,29 @@ typedef struct {
         uint64_t uint64;
         opal_byte_object_t bo;
         float fval;
+        double dval;
         struct timeval tv;
+        opal_process_name_t name;
+        opal_bool_array_t flag_array;
+        opal_uint8_array_t byte_array;
+        opal_string_array_t string_array;
+        opal_size_array_t size_array;
+        opal_int_array_t integer_array;
+        opal_int8_array_t int8_array;
+        opal_int16_array_t int16_array;
+        opal_int32_array_t int32_array;
+        opal_int64_array_t int64_array;
+        opal_uint_array_t uint_array;
+        opal_uint8_array_t uint8_array;
+        opal_uint16_array_t uint16_array;
+        opal_uint32_array_t uint32_array;
+        opal_uint64_array_t uint64_array;
+        opal_byte_object_array_t bo_array;
+        opal_float_array_t fval_array;
+        opal_double_array_t dval_array;
+        opal_pid_array_t pid_array;
+        opal_timeval_array_t tv_array;
+        void *ptr;  // never packed or passed anywhere
     } data;
 } opal_value_t;
 OPAL_DECLSPEC OBJ_CLASS_DECLARATION(opal_value_t);
@@ -246,7 +379,7 @@ struct opal_buffer_t {
     /** Where the next data will be unpacked from (within the
         allocated memory starting as base_ptr) */
     char *unpack_ptr;
-    
+
     /** Number of bytes allocated (starting at base_ptr) */
     size_t bytes_allocated;
     /** Number of bytes used by the buffer (i.e., amount of data --
