@@ -6,15 +6,17 @@
  * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -42,7 +44,7 @@
  * Public variables
  */
 char *mca_base_component_path = NULL;
-bool mca_base_opened = false;
+int mca_base_opened = 0;
 char *mca_base_system_default_path = NULL;
 char *mca_base_user_default_path = NULL;
 bool mca_base_component_show_load_errors = true;
@@ -58,27 +60,25 @@ static void parse_verbose(char *e, opal_output_stream_t *lds);
 
 
 /*
- * Main MCA initialization.  
+ * Main MCA initialization.
  */
 int mca_base_open(void)
 {
     char *value;
     opal_output_stream_t lds;
-    char hostname[64];
+    char hostname[OPAL_MAXHOSTNAMELEN];
     int var_id;
 
-    if (!mca_base_opened) {
-        mca_base_opened = true;
-    } else {
+    if (mca_base_opened++) {
         return OPAL_SUCCESS;
     }
 
     /* define the system and user default paths */
 #if OPAL_WANT_HOME_CONFIG_FILES
-    mca_base_system_default_path = strdup(opal_install_dirs.ompilibdir);
+    mca_base_system_default_path = strdup(opal_install_dirs.opallibdir);
     asprintf(&mca_base_user_default_path, "%s"OPAL_PATH_SEP".openmpi"OPAL_PATH_SEP"components", opal_home_directory());
 #else
-    asprintf(&mca_base_system_default_path, "%s", opal_install_dirs.ompilibdir); 
+    asprintf(&mca_base_system_default_path, "%s", opal_install_dirs.opallibdir);
 #endif
 
     /* see if the user wants to override the defaults */
@@ -91,7 +91,7 @@ int mca_base_open(void)
 
     mca_base_component_path = value;
     var_id = mca_base_var_register("opal", "mca", "base", "component_path",
-                                   "Path where to look for Open MPI and ORTE components",
+                                   "Path where to look for additional components",
                                    MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
                                    OPAL_INFO_LVL_9,
                                    MCA_BASE_VAR_SCOPE_READONLY,
@@ -137,10 +137,10 @@ int mca_base_open(void)
     } else {
         set_defaults(&lds);
     }
-    gethostname(hostname, 64);
+    gethostname(hostname, sizeof(hostname));
     asprintf(&lds.lds_prefix, "[%s:%05d] ", hostname, getpid());
     opal_output_reopen(0, &lds);
-    opal_output_verbose(5, 0, "mca: base: opening components");
+    opal_output_verbose (MCA_BASE_VERBOSE_COMPONENT, 0, "mca: base: opening components");
     free(lds.lds_prefix);
 
     /* Open up the component repository */
@@ -158,9 +158,7 @@ static void set_defaults(opal_output_stream_t *lds)
     /* Load up defaults */
 
     OBJ_CONSTRUCT(lds, opal_output_stream_t);
-#ifndef __WINDOWS__
     lds->lds_syslog_priority = LOG_INFO;
-#endif
     lds->lds_syslog_ident = "ompi";
     lds->lds_want_stderr = true;
 }
@@ -171,85 +169,83 @@ static void set_defaults(opal_output_stream_t *lds)
  */
 static void parse_verbose(char *e, opal_output_stream_t *lds)
 {
-  char *edup;
-  char *ptr, *next;
-  bool have_output = false;
+    char *edup;
+    char *ptr, *next;
+    bool have_output = false;
 
-  if (NULL == e) {
-    return;
-  }
-
-  edup = strdup(e);
-  ptr = edup;
-
-  /* Now parse the environment variable */
-
-  while (NULL != ptr && strlen(ptr) > 0) {
-    next = strchr(ptr, ',');
-    if (NULL != next) {
-      *next = '\0';
+    if (NULL == e) {
+        return;
     }
 
-    if (0 == strcasecmp(ptr, "syslog")) {
-#ifndef __WINDOWS__  /* there is no syslog */
-		lds->lds_want_syslog = true;
-      have_output = true;
-    }
-    else if (strncasecmp(ptr, "syslogpri:", 10) == 0) {
-      lds->lds_want_syslog = true;
-      have_output = true;
-      if (strcasecmp(ptr + 10, "notice") == 0)
-	lds->lds_syslog_priority = LOG_NOTICE;
-      else if (strcasecmp(ptr + 10, "INFO") == 0)
-	lds->lds_syslog_priority = LOG_INFO;
-      else if (strcasecmp(ptr + 10, "DEBUG") == 0)
-	lds->lds_syslog_priority = LOG_DEBUG;
-    } else if (strncasecmp(ptr, "syslogid:", 9) == 0) {
-      lds->lds_want_syslog = true;
-      lds->lds_syslog_ident = ptr + 9;
-#endif
+    edup = strdup(e);
+    ptr = edup;
+
+    /* Now parse the environment variable */
+
+    while (NULL != ptr && strlen(ptr) > 0) {
+        next = strchr(ptr, ',');
+        if (NULL != next) {
+            *next = '\0';
+        }
+
+        if (0 == strcasecmp(ptr, "syslog")) {
+            lds->lds_want_syslog = true;
+            have_output = true;
+        }
+        else if (strncasecmp(ptr, "syslogpri:", 10) == 0) {
+            lds->lds_want_syslog = true;
+            have_output = true;
+            if (strcasecmp(ptr + 10, "notice") == 0)
+                lds->lds_syslog_priority = LOG_NOTICE;
+            else if (strcasecmp(ptr + 10, "INFO") == 0)
+                lds->lds_syslog_priority = LOG_INFO;
+            else if (strcasecmp(ptr + 10, "DEBUG") == 0)
+                lds->lds_syslog_priority = LOG_DEBUG;
+        } else if (strncasecmp(ptr, "syslogid:", 9) == 0) {
+            lds->lds_want_syslog = true;
+            lds->lds_syslog_ident = ptr + 9;
+        }
+
+        else if (strcasecmp(ptr, "stdout") == 0) {
+            lds->lds_want_stdout = true;
+            have_output = true;
+        } else if (strcasecmp(ptr, "stderr") == 0) {
+            lds->lds_want_stderr = true;
+            have_output = true;
+        }
+
+        else if (strcasecmp(ptr, "file") == 0 || strcasecmp(ptr, "file:") == 0) {
+            lds->lds_want_file = true;
+            have_output = true;
+        } else if (strncasecmp(ptr, "file:", 5) == 0) {
+            lds->lds_want_file = true;
+            lds->lds_file_suffix = strdup(ptr + 5);
+            have_output = true;
+        } else if (strcasecmp(ptr, "fileappend") == 0) {
+            lds->lds_want_file = true;
+            lds->lds_want_file_append = 1;
+            have_output = true;
+        }
+
+        else if (strncasecmp(ptr, "level", 5) == 0) {
+            lds->lds_verbose_level = 0;
+            if (ptr[5] == OPAL_ENV_SEP)
+                lds->lds_verbose_level = atoi(ptr + 6);
+        }
+
+        if (NULL == next) {
+            break;
+        }
+        ptr = next + 1;
     }
 
-    else if (strcasecmp(ptr, "stdout") == 0) {
-      lds->lds_want_stdout = true;
-      have_output = true;
-    } else if (strcasecmp(ptr, "stderr") == 0) {
-      lds->lds_want_stderr = true;
-      have_output = true;
+    /* If we didn't get an output, default to stderr */
+
+    if (!have_output) {
+        lds->lds_want_stderr = true;
     }
 
-    else if (strcasecmp(ptr, "file") == 0) {
-      lds->lds_want_file = true;
-      have_output = true;
-    } else if (strncasecmp(ptr, "file:", 5) == 0) {
-      lds->lds_want_file = true;
-      lds->lds_file_suffix = ptr + 5;
-      have_output = true;
-    } else if (strcasecmp(ptr, "fileappend") == 0) {
-      lds->lds_want_file = true;
-      lds->lds_want_file_append = 1;
-      have_output = true;
-    } 
+    /* All done */
 
-    else if (strncasecmp(ptr, "level", 5) == 0) {
-      lds->lds_verbose_level = 0;
-      if (ptr[5] == OPAL_ENV_SEP)
-        lds->lds_verbose_level = atoi(ptr + 6);
-    }
-
-    if (NULL == next) {
-      break;
-    }
-    ptr = next + 1;
-  }
-
-  /* If we didn't get an output, default to stderr */
-
-  if (!have_output) {
-    lds->lds_want_stderr = true;
-  }
-
-  /* All done */
-
-  free(edup);
+    free(edup);
 }
