@@ -1,7 +1,8 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2013-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2015 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -75,7 +76,7 @@ int mca_base_pvar_find (const char *project, const char *framework, const char *
         return OPAL_ERROR;
     }
 
-    ret = mca_base_pvar_find_by_name (full_name, &index);
+    ret = mca_base_pvar_find_by_name (full_name, MCA_BASE_PVAR_CLASS_ANY, &index);
     free (full_name);
 
     /* NTH: should we verify the name components match the returned variable? */
@@ -83,8 +84,9 @@ int mca_base_pvar_find (const char *project, const char *framework, const char *
     return (OPAL_SUCCESS != ret) ? ret : index;
 }
 
-int mca_base_pvar_find_by_name (const char *full_name, int *index)
+int mca_base_pvar_find_by_name (const char *full_name, int var_class, int *index)
 {
+    mca_base_pvar_t *pvar;
     void *tmp;
     int rc;
 
@@ -94,6 +96,15 @@ int mca_base_pvar_find_by_name (const char *full_name, int *index)
         return rc;
     }
 
+    rc = mca_base_pvar_get_internal ((int)(uintptr_t) tmp, &pvar, false);
+    if (OPAL_SUCCESS != rc) {
+        return rc;
+    }
+
+    if (MCA_BASE_PVAR_CLASS_ANY != var_class && pvar->var_class != var_class) {
+        return OPAL_ERR_NOT_FOUND;
+    }
+
     *index = (int)(uintptr_t) tmp;
 
     return OPAL_SUCCESS;
@@ -113,6 +124,8 @@ int mca_base_pvar_finalize (void)
             }
         }
 
+        pvar_count = 0;
+
         OBJ_DESTRUCT(&registered_pvars);
         OBJ_DESTRUCT(&mca_base_pvar_index_hash);
     }
@@ -166,7 +179,7 @@ int mca_base_pvar_register (const char *project, const char *framework, const ch
                             int bind, mca_base_pvar_flag_t flags, mca_base_get_value_fn_t get_value,
                             mca_base_set_value_fn_t set_value, mca_base_notify_fn_t notify, void *ctx)
 {
-    int ret, group_index;
+    int ret, group_index, pvar_index;
     mca_base_pvar_t *pvar;
 
     /* assert on usage errors */
@@ -257,8 +270,8 @@ int mca_base_pvar_register (const char *project, const char *framework, const ch
 
         do {
             /* generate the variable's full name */
-            mca_base_var_generate_full_name4 (NULL, framework, component, name, &pvar->name);
-            if (NULL == pvar->name) {
+            ret = mca_base_var_generate_full_name4 (NULL, framework, component, name, &pvar->name);
+            if (OPAL_SUCCESS != ret) {
                 ret = OPAL_ERR_OUT_OF_RESOURCE;
                 break;
             }
@@ -271,15 +284,18 @@ int mca_base_pvar_register (const char *project, const char *framework, const ch
                 }
             }
 
-            /* add this performance variable to the MCA variable group */
-            ret = mca_base_var_group_add_pvar (group_index, pvar_count);
-            if (0 > ret) {
+            pvar_index = opal_pointer_array_add (&registered_pvars, pvar);
+            if (0 > pvar_index) {
                 break;
             }
+            pvar->pvar_index = pvar_index;
 
-            ret = opal_pointer_array_add (&registered_pvars, pvar);
-            if (0 > ret) {
-                break;
+            /* add this performance variable to the MCA variable group */
+            if (0 <= group_index) {
+                ret = mca_base_var_group_add_pvar (group_index, pvar_index);
+                if (0 > ret) {
+                    break;
+                }
             }
 
             opal_hash_table_set_value_ptr (&mca_base_pvar_index_hash, pvar->name, strlen (pvar->name),
@@ -314,7 +330,7 @@ int mca_base_pvar_register (const char *project, const char *framework, const ch
     if (!(flags & MCA_BASE_PVAR_FLAG_READONLY)) {
         pvar->set_value = set_value ? set_value : mca_base_pvar_default_set_value;
     }
-        
+
     pvar->ctx        = ctx;
     pvar->pvar_index = pvar_count;
 
@@ -327,9 +343,8 @@ int mca_base_component_pvar_register (const mca_base_component_t *component, con
                                       int bind, mca_base_pvar_flag_t flags, mca_base_get_value_fn_t get_value,
                                       mca_base_set_value_fn_t set_value, mca_base_notify_fn_t notify, void *ctx)
 {
-    /* XXX -- component_update -- We will stash the project name in the component */
     /* invalidate this variable if the component's group is deregistered */
-    return mca_base_pvar_register(NULL, component->mca_type_name, component->mca_component_name,
+    return mca_base_pvar_register(component->mca_project_name, component->mca_type_name, component->mca_component_name,
                                   name, description, verbosity, var_class, type, enumerator, bind,
                                   flags | MCA_BASE_PVAR_FLAG_IWG, get_value, set_value, notify, ctx);
 }
@@ -432,7 +447,7 @@ int mca_base_pvar_handle_alloc (mca_base_pvar_session_t *session, int index, voi
             obj_handle = NULL;
         } else if (0 != pvar->bind && NULL == obj_handle) {
             /* this is an application error. what is the correct error code? */
-            ret = OPAL_ERROR;
+            ret = OPAL_ERR_BAD_PARAM;
             break;
         }
 
@@ -444,7 +459,7 @@ int mca_base_pvar_handle_alloc (mca_base_pvar_session_t *session, int index, voi
         }
 
         pvar_handle->obj_handle = obj_handle;
-        pvar_handle->pvar = pvar; 
+        pvar_handle->pvar = pvar;
 
         *handle = pvar_handle;
 
@@ -489,7 +504,7 @@ int mca_base_pvar_handle_alloc (mca_base_pvar_session_t *session, int index, voi
                 ret = OPAL_ERR_OUT_OF_RESOURCE;
                 break;
             }
-       
+
             pvar_handle->last_value = calloc (*count, datatype_size);
             if (NULL == pvar_handle->last_value) {
                 ret = OPAL_ERR_OUT_OF_RESOURCE;
@@ -696,7 +711,7 @@ int mca_base_pvar_handle_write_value (mca_base_pvar_handle_t *handle, const void
     if (OPAL_SUCCESS != ret) {
         return ret;
     }
-    
+
     memmove (handle->current_value, value, handle->count * var_type_sizes[handle->pvar->type]);
 
     return OPAL_SUCCESS;
@@ -829,17 +844,16 @@ int mca_base_pvar_dump(int index, char ***out, mca_base_var_dump_type_t output_t
         }
 
         /* build the message*/
-        asprintf(&tmp, "mca:%s:%s:pvar:%s:", framework, component,
-                 full_name);
+        (void)asprintf(&tmp, "mca:%s:%s:pvar:%s:", framework, component, full_name);
 
-        asprintf(out[0] + line++, "%sclass:%s", tmp, pvar_class_names[pvar->var_class]);
-        asprintf(out[0] + line++, "%sread-only:%s", tmp, mca_base_pvar_is_readonly(pvar) ? "true" : "false");
-        asprintf(out[0] + line++, "%scontinuous:%s", tmp, mca_base_pvar_is_continuous(pvar) ? "true" : "false");
-        asprintf(out[0] + line++, "%satomic:%s", tmp, mca_base_pvar_is_atomic(pvar) ? "true" : "false");
+        (void)asprintf(out[0] + line++, "%sclass:%s", tmp, pvar_class_names[pvar->var_class]);
+        (void)asprintf(out[0] + line++, "%sread-only:%s", tmp, mca_base_pvar_is_readonly(pvar) ? "true" : "false");
+        (void)asprintf(out[0] + line++, "%scontinuous:%s", tmp, mca_base_pvar_is_continuous(pvar) ? "true" : "false");
+        (void)asprintf(out[0] + line++, "%satomic:%s", tmp, mca_base_pvar_is_atomic(pvar) ? "true" : "false");
 
         /* if it has a help message, output the help message */
         if (pvar->description) {
-            asprintf(out[0] + line++, "%shelp:%s", tmp, pvar->description);
+            (void)asprintf(out[0] + line++, "%shelp:%s", tmp, pvar->description);
         }
 
         if (NULL != pvar->enumerator) {
@@ -853,11 +867,12 @@ int mca_base_pvar_dump(int index, char ***out, mca_base_var_dump_type_t output_t
                     continue;
                 }
 
-                asprintf(out[0] + line++, "%senumerator:value:%d:%s", tmp, enum_value, enum_string);
+                (void)asprintf(out[0] + line++, "%senumerator:value:%d:%s", tmp, enum_value, enum_string);
             }
         }
 
-        asprintf(out[0] + line++, "%stype:%s", tmp, var_type_names[pvar->type]);
+        (void)asprintf(out[0] + line++, "%stype:%s", tmp, var_type_names[pvar->type]);
+        free(tmp);  // release tmp storage
     } else {
         /* there will be at most three lines in the pretty print case */
         *out = (char **) calloc (3, sizeof (char *));
@@ -865,11 +880,11 @@ int mca_base_pvar_dump(int index, char ***out, mca_base_var_dump_type_t output_t
             return OPAL_ERR_OUT_OF_RESOURCE;
         }
 
-        asprintf (out[0] + line++, "performance \"%s\" (type: %s, class: %s)", full_name,
-                  var_type_names[pvar->type], pvar_class_names[pvar->var_class]);
+        (void)asprintf (out[0] + line++, "performance \"%s\" (type: %s, class: %s)", full_name,
+                        var_type_names[pvar->type], pvar_class_names[pvar->var_class]);
 
         if (pvar->description) {
-            asprintf(out[0] + line++, "%s", pvar->description);
+            (void)asprintf(out[0] + line++, "%s", pvar->description);
         }
 
         if (NULL != pvar->enumerator) {
@@ -877,7 +892,7 @@ int mca_base_pvar_dump(int index, char ***out, mca_base_var_dump_type_t output_t
 
             ret = pvar->enumerator->dump(pvar->enumerator, &values);
             if (OPAL_SUCCESS == ret) {
-                asprintf (out[0] + line++, "Values: %s", values);
+                (void)asprintf (out[0] + line++, "Values: %s", values);
                 free (values);
             }
         }
@@ -913,12 +928,12 @@ static void mca_base_pvar_destructor (mca_base_pvar_t *pvar)
 OBJ_CLASS_INSTANCE(mca_base_pvar_t, opal_object_t, mca_base_pvar_contructor, mca_base_pvar_destructor);
 
 /* mca_base_pvar_session_t class */
-static void ompi_mpi_pvar_session_constructor (mca_base_pvar_session_t *session)
+static void opal_mpi_pvar_session_constructor (mca_base_pvar_session_t *session)
 {
     OBJ_CONSTRUCT(&session->handles, opal_list_t);
 }
 
-static void ompi_mpi_pvar_session_destructor (mca_base_pvar_session_t *session)
+static void opal_mpi_pvar_session_destructor (mca_base_pvar_session_t *session)
 {
     mca_base_pvar_handle_t *handle, *next;
 
@@ -932,8 +947,8 @@ static void ompi_mpi_pvar_session_destructor (mca_base_pvar_session_t *session)
     OBJ_DESTRUCT(&session->handles);
 }
 
-OBJ_CLASS_INSTANCE(mca_base_pvar_session_t, opal_object_t, ompi_mpi_pvar_session_constructor,
-                   ompi_mpi_pvar_session_destructor);
+OBJ_CLASS_INSTANCE(mca_base_pvar_session_t, opal_object_t, opal_mpi_pvar_session_constructor,
+                   opal_mpi_pvar_session_destructor);
 
 /* mca_base_pvar_handle_t class */
 static void mca_base_pvar_handle_constructor (mca_base_pvar_handle_t *handle)
@@ -962,7 +977,9 @@ static void mca_base_pvar_handle_destructor (mca_base_pvar_handle_t *handle)
     }
 
     /* remove this handle from the pvar list */
-    opal_list_remove_item (&handle->pvar->bound_handles, &handle->list2);
+    if (handle->pvar) {
+        opal_list_remove_item (&handle->pvar->bound_handles, &handle->list2);
+    }
 
     OBJ_DESTRUCT(&handle->list2);
 
