@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,19 +6,22 @@
  * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2010-2011 Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2011-2012 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2011-2013 Los Alamos National Security, LLC.  All rights
- *                         reserved. 
- * Copyright (c) 2013      Intel, Inc.  All rights reserved. 
+ * Copyright (c) 2011-2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2011-2017 Los Alamos National Security, LLC.  All rights
+ *                         reserved.
+ * Copyright (c) 2013-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2017      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  *
  */
@@ -37,17 +41,18 @@
 #include "opal/hash_string.h"
 #include "opal/class/opal_hash_table.h"
 #include "opal/class/opal_list.h"
-#include "opal/mca/db/base/base.h"
 #include "opal/mca/event/event.h"
 #include "opal/runtime/opal.h"
-#include "opal/runtime/opal_cr.h"
 
+#include "opal/util/arch.h"
 #include "opal/util/argv.h"
 #include "opal/util/if.h"
 #include "opal/util/os_path.h"
 #include "opal/util/output.h"
 #include "opal/util/malloc.h"
 #include "opal/util/basename.h"
+#include "opal/util/fd.h"
+#include "opal/mca/pmix/base/base.h"
 #include "opal/mca/pstat/base/base.h"
 #include "opal/mca/hwloc/base/base.h"
 
@@ -56,6 +61,7 @@
 #include "orte/mca/rml/rml_types.h"
 #include "orte/mca/routed/base/base.h"
 #include "orte/mca/routed/routed.h"
+#include "orte/mca/rtc/base/base.h"
 #include "orte/mca/dfs/base/base.h"
 #include "orte/mca/errmgr/base/base.h"
 #include "orte/mca/grpcomm/base/base.h"
@@ -64,16 +70,14 @@
 #include "orte/mca/plm/base/base.h"
 #include "orte/mca/plm/plm.h"
 #include "orte/mca/odls/base/base.h"
-#include "orte/mca/sensor/base/base.h"
-#include "orte/mca/sensor/sensor.h"
 #include "orte/mca/rmaps/base/base.h"
-#if OPAL_ENABLE_FT_CR == 1
-#include "orte/mca/snapc/base/base.h"
-#endif
 #include "orte/mca/filem/base/base.h"
+#include "orte/mca/schizo/base/base.h"
 #include "orte/mca/state/base/base.h"
 #include "orte/mca/state/state.h"
 
+#include "orte/orted/pmix/pmix_server.h"
+
 #include "orte/util/show_help.h"
 #include "orte/util/proc_info.h"
 #include "orte/util/session_dir.h"
@@ -81,13 +85,11 @@
 #include "orte/util/name_fns.h"
 #include "orte/util/show_help.h"
 #include "orte/util/comm/comm.h"
-#include "orte/util/nidmap.h"
 
 #include "orte/runtime/runtime.h"
 #include "orte/runtime/orte_wait.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/runtime/orte_quit.h"
-#include "orte/runtime/orte_cr.h"
 #include "orte/runtime/orte_locks.h"
 #include "orte/runtime/orte_data_server.h"
 
@@ -112,10 +114,7 @@ static bool forcibly_die=false;
 static opal_event_t term_handler;
 static opal_event_t epipe_handler;
 static int term_pipe[2];
-static opal_event_t sigusr1_handler;
-static opal_event_t sigusr2_handler;
-static opal_event_t sigtstp_handler;
-static opal_event_t sigcont_handler;
+static opal_event_t *forward_signals_events = NULL;
 
 static void abort_signal_callback(int signal);
 static void clean_abort(int fd, short flags, void *arg);
@@ -139,6 +138,12 @@ static int rte_init(void)
     orte_node_t *node;
     orte_proc_t *proc;
     orte_app_context_t *app;
+    char **aliases, *aptr;
+    char *coprocessors, **sns;
+    uint32_t h;
+    int idx;
+    orte_topology_t *t;
+    ess_hnp_signal_t *sig;
 
     /* run the prolog */
     if (ORTE_SUCCESS != (ret = orte_ess_base_std_prolog())) {
@@ -167,62 +172,51 @@ static int rte_init(void)
     opal_event_set(orte_event_base, &term_handler, term_pipe[0], OPAL_EV_READ, clean_abort, NULL);
     opal_event_set_priority(&term_handler, ORTE_ERROR_PRI);
     opal_event_add(&term_handler, NULL);
+
+    /* Set both ends of this pipe to be close-on-exec so that no
+       children inherit it */
+    if (opal_fd_set_cloexec(term_pipe[0]) != OPAL_SUCCESS ||
+        opal_fd_set_cloexec(term_pipe[1]) != OPAL_SUCCESS) {
+        error = "unable to set the pipe to CLOEXEC";
+        goto error;
+    }
+
     /* point the signal trap to a function that will activate that event */
     signal(SIGTERM, abort_signal_callback);
     signal(SIGINT, abort_signal_callback);
     signal(SIGHUP, abort_signal_callback);
 
-    /** setup callbacks for signals we should foward */
-    setup_sighandler(SIGUSR1, &sigusr1_handler, signal_forward_callback);
-    setup_sighandler(SIGUSR2, &sigusr2_handler, signal_forward_callback);
-    setup_sighandler(SIGTSTP, &sigtstp_handler, signal_forward_callback);
-    setup_sighandler(SIGCONT, &sigcont_handler, signal_forward_callback);
-    signals_set = true;
-
-#if OPAL_HAVE_HWLOC
-    {
-        hwloc_obj_t obj;
-        unsigned i, j;
-
-        /* get the local topology */
-        if (NULL == opal_hwloc_topology) {
-            if (OPAL_SUCCESS != opal_hwloc_base_get_topology()) {
-                error = "topology discovery";
-                goto error;
-            }
+    /** setup callbacks for signals we should forward */
+    if (0 < (idx = opal_list_get_size(&mca_ess_hnp_component.signals))) {
+        forward_signals_events = (opal_event_t*)malloc(sizeof(opal_event_t) * idx);
+        if (NULL == forward_signals_events) {
+            ret = ORTE_ERR_OUT_OF_RESOURCE;
+            error = "unable to malloc";
+            goto error;
         }
-
-        /* remove the hostname from the topology. Unfortunately, hwloc
-         * decided to add the source hostname to the "topology", thus
-         * rendering it unusable as a pure topological description. So
-         * we remove that information here.
-         */
-        obj = hwloc_get_root_obj(opal_hwloc_topology);
-        for (i=0; i < obj->infos_count; i++) {
-            if (NULL == obj->infos[i].name ||
-                NULL == obj->infos[i].value) {
-                continue;
-            }
-            if (0 == strncmp(obj->infos[i].name, "HostName", strlen("HostName"))) {
-                free(obj->infos[i].name);
-                free(obj->infos[i].value);
-                /* left justify the array */
-                for (j=i; j < obj->infos_count-1; j++) {
-                    obj->infos[j] = obj->infos[j+1];
-                }
-                obj->infos[obj->infos_count-1].name = NULL;
-                obj->infos[obj->infos_count-1].value = NULL;
-                obj->infos_count--;
-                break;
-            }
-        }
-
-        if (4 < opal_output_get_verbosity(orte_ess_base_framework.framework_output)) {
-            opal_output(0, "%s Topology Info:", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
-            opal_dss.dump(0, opal_hwloc_topology, OPAL_HWLOC_TOPO);
+        idx = 0;
+        OPAL_LIST_FOREACH(sig, &mca_ess_hnp_component.signals, ess_hnp_signal_t) {
+            setup_sighandler(sig->signal, forward_signals_events + idx, signal_forward_callback);
+            ++idx;
         }
     }
-#endif
+    signals_set = true;
+
+    /* get the local topology */
+    if (NULL == opal_hwloc_topology) {
+        if (OPAL_SUCCESS != (ret = opal_hwloc_base_get_topology())) {
+            error = "topology discovery";
+            goto error;
+        }
+    }
+    /* generate the signature */
+    orte_topo_signature = opal_hwloc_base_get_topo_signature(opal_hwloc_topology);
+
+    if (15 < opal_output_get_verbosity(orte_ess_base_framework.framework_output)) {
+        opal_output(0, "%s Topology Info:", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+        opal_dss.dump(0, opal_hwloc_topology, OPAL_HWLOC_TOPO);
+    }
+
 
     /* if we are using xml for output, put an mpirun start tag */
     if (orte_xml_output) {
@@ -230,41 +224,30 @@ static int rte_init(void)
         fflush(orte_xml_fp);
     }
 
-    /* setup the global nidmap/pidmap object */
-    orte_nidmap.bytes = NULL;
-    orte_nidmap.size = 0;
-    orte_pidmap.bytes = NULL;
-    orte_pidmap.size = 0;
-
     /* open and setup the opal_pstat framework so we can provide
      * process stats if requested
      */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&opal_pstat_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "opal_pstat_base_open";
         goto error;
     }
     if (ORTE_SUCCESS != (ret = opal_pstat_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "opal_pstat_base_select";
         goto error;
     }
-  
+
     /* open and setup the state machine */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_state_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_state_base_open";
         goto error;
     }
     if (ORTE_SUCCESS != (ret = orte_state_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_state_base_select";
         goto error;
     }
 
     /* open the errmgr */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_errmgr_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_errmgr_base_open";
         goto error;
     }
@@ -275,43 +258,81 @@ static int rte_init(void)
      * first and select that component.
      */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_plm_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_plm_base_open";
         goto error;
     }
-    
+
     if (ORTE_SUCCESS != (ret = orte_plm_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_plm_base_select";
+        if (ORTE_ERR_FATAL == ret) {
+            /* we already output a show_help - so keep down the verbage */
+            ret = ORTE_ERR_SILENT;
+        }
         goto error;
     }
     /* if we were spawned by a singleton, our jobid was given to us */
     if (NULL != orte_ess_base_jobid) {
         if (ORTE_SUCCESS != (ret = orte_util_convert_string_to_jobid(&ORTE_PROC_MY_NAME->jobid, orte_ess_base_jobid))) {
-            ORTE_ERROR_LOG(ret);
             error = "convert_string_to_jobid";
             goto error;
         }
         ORTE_PROC_MY_NAME->vpid = 0;
     } else {
         if (ORTE_SUCCESS != (ret = orte_plm.set_hnp_name())) {
-            ORTE_ERROR_LOG(ret);
             error = "orte_plm_set_hnp_name";
             goto error;
         }
     }
+    /* now that my name is set, xfer it to the OPAL layer */
+    orte_process_info.super.proc_name = *(opal_process_name_t*)ORTE_PROC_MY_NAME;
+    orte_process_info.super.proc_hostname = strdup(orte_process_info.nodename);
+    orte_process_info.super.proc_flags = OPAL_PROC_ALL_LOCAL;
+    orte_process_info.super.proc_arch = opal_local_arch;
+    opal_proc_local_set(&orte_process_info.super);
+
+    /* setup my session directory here as the OOB may need it */
+    if (orte_create_session_dirs) {
+        OPAL_OUTPUT_VERBOSE((2, orte_debug_output,
+                             "%s setting up session dir with\n\ttmpdir: %s\n\thost %s",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                             (NULL == orte_process_info.tmpdir_base) ? "UNDEF" : orte_process_info.tmpdir_base,
+                             orte_process_info.nodename));
+
+        /* take a pass thru the session directory code to fillin the
+         * tmpdir names - don't create anything yet
+         */
+        if (ORTE_SUCCESS != (ret = orte_session_dir(false,
+                                                    orte_process_info.tmpdir_base,
+                                                    orte_process_info.nodename, NULL,
+                                                    ORTE_PROC_MY_NAME))) {
+            error = "orte_session_dir define";
+            goto error;
+        }
+        /* clear the session directory just in case there are
+         * stale directories laying around
+         */
+        orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
+
+        /* now actually create the directory tree */
+        if (ORTE_SUCCESS != (ret = orte_session_dir(true,
+                                                    orte_process_info.tmpdir_base,
+                                                    orte_process_info.nodename, NULL,
+                                                    ORTE_PROC_MY_NAME))) {
+            error = "orte_session_dir";
+            goto error;
+        }
+    }
+
     /* Setup the communication infrastructure */
-    
+
     /*
      * OOB Layer
      */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_oob_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_oob_base_open";
         goto error;
     }
     if (ORTE_SUCCESS != (ret = orte_oob_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_oob_base_select";
         goto error;
     }
@@ -320,22 +341,19 @@ static int rte_init(void)
      * Runtime Messaging Layer
      */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_rml_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_rml_base_open";
         goto error;
     }
     if (ORTE_SUCCESS != (ret = orte_rml_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_rml_base_select";
         goto error;
     }
 
     if (ORTE_SUCCESS != (ret = orte_errmgr_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_errmgr_base_select";
         goto error;
     }
-    
+
     /* setup the global job and node arrays */
     orte_job_data = OBJ_NEW(opal_pointer_array_t);
     if (ORTE_SUCCESS != (ret = opal_pointer_array_init(orte_job_data,
@@ -346,7 +364,7 @@ static int rte_init(void)
         error = "setup job array";
         goto error;
     }
-    
+
     orte_node_pool = OBJ_NEW(opal_pointer_array_t);
     if (ORTE_SUCCESS != (ret = opal_pointer_array_init(orte_node_pool,
                                                        ORTE_GLOBAL_ARRAY_BLOCK_SIZE,
@@ -366,10 +384,7 @@ static int rte_init(void)
         goto error;
     }
 
-    /* init the nidmap - just so we register that verbosity */
-    orte_util_nidmap_init(NULL);
-
-    /* Setup the job data object for the daemons */        
+    /* Setup the job data object for the daemons */
     /* create and store the job data object */
     jdata = OBJ_NEW(orte_job_t);
     jdata->jobid = ORTE_PROC_MY_NAME->jobid;
@@ -379,7 +394,7 @@ static int rte_init(void)
      * are running!
      */
     jdata->state = ORTE_JOB_STATE_DAEMONS_REPORTED;
-   
+
     /* every job requires at least one app */
     app = OBJ_NEW(orte_app_context_t);
     opal_pointer_array_set_item(jdata->apps, 0, app);
@@ -389,25 +404,26 @@ static int rte_init(void)
     node = OBJ_NEW(orte_node_t);
     node->name = strdup(orte_process_info.nodename);
     node->index = opal_pointer_array_set_item(orte_node_pool, 0, node);
-#if OPAL_HAVE_HWLOC
+
     /* add it to the array of known topologies */
-    opal_pointer_array_add(orte_node_topologies, opal_hwloc_topology);
-#endif
+    t = OBJ_NEW(orte_topology_t);
+    t->topo = opal_hwloc_topology;
+    t->sig = strdup(orte_topo_signature);
+    opal_pointer_array_add(orte_node_topologies, t);
 
     /* create and store a proc object for us */
     proc = OBJ_NEW(orte_proc_t);
     proc->name.jobid = ORTE_PROC_MY_NAME->jobid;
     proc->name.vpid = ORTE_PROC_MY_NAME->vpid;
-    
+
     proc->pid = orte_process_info.pid;
     proc->rml_uri = orte_rml.get_contact_info();
     proc->state = ORTE_PROC_STATE_RUNNING;
     OBJ_RETAIN(node);  /* keep accounting straight */
     proc->node = node;
-    proc->nodename = node->name;
     opal_pointer_array_set_item(jdata->procs, proc->name.vpid, proc);
 
-    /* record that the daemon (i.e., us) is on this node 
+    /* record that the daemon (i.e., us) is on this node
      * NOTE: we do not add the proc object to the node's
      * proc array because we are not an application proc.
      * Instead, we record it in the daemon field of the
@@ -415,14 +431,19 @@ static int rte_init(void)
      */
     OBJ_RETAIN(proc);   /* keep accounting straight */
     node->daemon = proc;
-    node->daemon_launched = true;
+    ORTE_FLAG_SET(node, ORTE_NODE_FLAG_DAEMON_LAUNCHED);
     node->state = ORTE_NODE_STATE_UP;
-    
+
     /* if we are to retain aliases, get ours */
     if (orte_retain_aliases) {
-        opal_ifgetaliases(&node->alias);
+        aliases = NULL;
+        opal_ifgetaliases(&aliases);
         /* add our own local name to it */
-        opal_argv_append_nosize(&node->alias, orte_process_info.nodename);
+        opal_argv_append_nosize(&aliases, orte_process_info.nodename);
+        aptr = opal_argv_join(aliases, ',');
+        opal_argv_free(aliases);
+        orte_set_attribute(&node->attributes, ORTE_NODE_ALIAS, ORTE_ATTR_LOCAL, aptr, OPAL_STRING);
+        free(aptr);
     }
 
     /* record that the daemon job is running */
@@ -444,20 +465,7 @@ static int rte_init(void)
         error = "orte_routed_base_select";
         goto error;
     }
-    
-    /* database */
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&opal_db_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_db_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = opal_db_base_select(true))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_db_base_select";
-        goto error;
-    }
-    /* set our id */
-    opal_db.set_id((opal_identifier_t*)ORTE_PROC_MY_NAME);
+
 
     /*
      * Group communications
@@ -494,65 +502,63 @@ static int rte_init(void)
         ORTE_ERROR_LOG(ret);
         error = "orte_ras_base_open";
         goto error;
-    }    
+    }
     if (ORTE_SUCCESS != (ret = orte_ras_base_select())) {
         ORTE_ERROR_LOG(ret);
         error = "orte_ras_base_find_available";
         goto error;
     }
-    
+
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_rmaps_base_framework, 0))) {
         ORTE_ERROR_LOG(ret);
         error = "orte_rmaps_base_open";
         goto error;
-    }    
+    }
     if (ORTE_SUCCESS != (ret = orte_rmaps_base_select())) {
         ORTE_ERROR_LOG(ret);
         error = "orte_rmaps_base_find_available";
         goto error;
     }
-#if OPAL_HAVE_HWLOC
-    {
-        char *coprocessors, **sns;
-        uint32_t h;
-        int idx;
 
-        /* if a topology file was given, then the rmaps framework open
-         * will have reset our topology. Ensure we always get the right
-         * one by setting our node topology afterwards
-         */
-        node->topology = opal_hwloc_topology;
+    /* if a topology file was given, then the rmaps framework open
+     * will have reset our topology. Ensure we always get the right
+     * one by setting our node topology afterwards
+     */
+    node->topology = opal_hwloc_topology;
 
-        /* init the hash table, if necessary */
-        if (NULL == orte_coprocessors) {
-            orte_coprocessors = OBJ_NEW(opal_hash_table_t);
-            opal_hash_table_init(orte_coprocessors, orte_process_info.num_procs);
+    /* init the hash table, if necessary */
+    if (NULL == orte_coprocessors) {
+        orte_coprocessors = OBJ_NEW(opal_hash_table_t);
+        opal_hash_table_init(orte_coprocessors, orte_process_info.num_procs);
+    }
+    /* detect and add any coprocessors */
+    coprocessors = opal_hwloc_base_find_coprocessors(opal_hwloc_topology);
+    if (NULL != coprocessors) {
+        /* separate the serial numbers of the coprocessors
+         * on this host
+         */
+        sns = opal_argv_split(coprocessors, ',');
+        for (idx=0; NULL != sns[idx]; idx++) {
+            /* compute the hash */
+            OPAL_HASH_STR(sns[idx], h);
+            /* mark that this coprocessor is hosted by this node */
+            opal_hash_table_set_value_uint32(orte_coprocessors, h, (void*)&(ORTE_PROC_MY_NAME->vpid));
         }
-        /* detect and add any coprocessors */
-        coprocessors = opal_hwloc_base_find_coprocessors(opal_hwloc_topology);
-        if (NULL != coprocessors) {
-            /* separate the serial numbers of the coprocessors
-             * on this host
-             */
-            sns = opal_argv_split(coprocessors, ',');
-            for (idx=0; NULL != sns[idx]; idx++) {
-                /* compute the hash */
-                OPAL_HASH_STR(sns[idx], h);
-                /* mark that this coprocessor is hosted by this node */
-                opal_hash_table_set_value_uint32(orte_coprocessors, h, (void*)&(ORTE_PROC_MY_NAME->vpid));
-            }
-            opal_argv_free(sns);
-            free(coprocessors);
-            orte_coprocessors_detected = true;
-        }
-        /* see if I am on a coprocessor */
-        coprocessors = opal_hwloc_base_check_on_coprocessor();
-        if (NULL != coprocessors) {
-            node->serial_number = coprocessors;
-            orte_coprocessors_detected = true;
-        }
+        opal_argv_free(sns);
+        free(coprocessors);
+        orte_coprocessors_detected = true;
+    }
+    /* see if I am on a coprocessor */
+    coprocessors = opal_hwloc_base_check_on_coprocessor();
+    if (NULL != coprocessors) {
+        /* compute the hash */
+        OPAL_HASH_STR(coprocessors, h);
+        /* mark that I am on this coprocessor */
+        opal_hash_table_set_value_uint32(orte_coprocessors, h, (void*)&(ORTE_PROC_MY_NAME->vpid));
+        orte_set_attribute(&node->attributes, ORTE_NODE_SERIAL_NUMBER, ORTE_ATTR_LOCAL, coprocessors, OPAL_STRING);
+        free(coprocessors);
+        orte_coprocessors_detected = true;
     }
-#endif
 
     /* Open/select the odls */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_odls_base_framework, 0))) {
@@ -565,7 +571,19 @@ static int rte_init(void)
         error = "orte_odls_base_select";
         goto error;
     }
-    
+
+    /* Open/select the rtc */
+    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_rtc_base_framework, 0))) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_rtc_base_open";
+        goto error;
+    }
+    if (ORTE_SUCCESS != (ret = orte_rtc_base_select())) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_rtc_base_select";
+        goto error;
+    }
+
     /* enable communication with the rml */
     if (ORTE_SUCCESS != (ret = orte_rml.enable_comm())) {
         ORTE_ERROR_LOG(ret);
@@ -579,61 +597,34 @@ static int rte_init(void)
 
     /* we are also officially a daemon, so better update that field too */
     orte_process_info.my_daemon_uri = strdup(orte_process_info.my_hnp_uri);
-    
+
     /* setup the orte_show_help system to recv remote output */
     orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD, ORTE_RML_TAG_SHOW_HELP,
                             ORTE_RML_PERSISTENT, orte_show_help_recv, NULL);
 
-    /* setup my session directory */
-    if (orte_create_session_dirs) {
-        OPAL_OUTPUT_VERBOSE((2, orte_debug_output,
-                             "%s setting up session dir with\n\ttmpdir: %s\n\thost %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             (NULL == orte_process_info.tmpdir_base) ? "UNDEF" : orte_process_info.tmpdir_base,
-                             orte_process_info.nodename));
-        
-        /* take a pass thru the session directory code to fillin the
-         * tmpdir names - don't create anything yet
-         */
-        if (ORTE_SUCCESS != (ret = orte_session_dir(false,
-                                                    orte_process_info.tmpdir_base,
-                                                    orte_process_info.nodename, NULL,
-                                                    ORTE_PROC_MY_NAME))) {
-            ORTE_ERROR_LOG(ret);
-            error = "orte_session_dir define";
-            goto error;
-        }
-        /* clear the session directory just in case there are
-         * stale directories laying around
-         */
-        orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
+    /* setup the data server */
+    if (ORTE_SUCCESS != (ret = orte_data_server_init())) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_data_server_init";
+        goto error;
+    }
 
-        /* now actually create the directory tree */
-        if (ORTE_SUCCESS != (ret = orte_session_dir(true,
-                                                    orte_process_info.tmpdir_base,
-                                                    orte_process_info.nodename, NULL,
-                                                    ORTE_PROC_MY_NAME))) {
-            ORTE_ERROR_LOG(ret);
-            error = "orte_session_dir";
-            goto error;
-        }
-        
-        /* Once the session directory location has been established, set
-           the opal_output hnp file location to be in the
-           proc-specific session directory. */
+    if (orte_create_session_dirs) {
+        /* set the opal_output hnp file location to be in the
+         * proc-specific session directory. */
         opal_output_set_output_file_info(orte_process_info.proc_session_dir,
                                          "output-", NULL, NULL);
-        
+
         /* save my contact info in a file for others to find */
         jobfam_dir = opal_dirname(orte_process_info.job_session_dir);
         contact_path = opal_os_path(false, jobfam_dir, "contact.txt", NULL);
         free(jobfam_dir);
-        
+
         OPAL_OUTPUT_VERBOSE((2, orte_debug_output,
                              "%s writing contact file %s",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                              contact_path));
-        
+
         if (ORTE_SUCCESS != (ret = orte_write_hnp_contact_file(contact_path))) {
             OPAL_OUTPUT_VERBOSE((2, orte_debug_output,
                                  "%s writing contact file failed with error %s",
@@ -647,15 +638,37 @@ static int rte_init(void)
         free(contact_path);
     }
 
+    /* setup the PMIx framework - ensure it skips all non-PMIx components */
+    putenv("OMPI_MCA_pmix=^s1,s2,cray");
+    if (OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_pmix_base_framework, 0))) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_pmix_base_open";
+        goto error;
+    }
+    if (ORTE_SUCCESS != (ret = opal_pmix_base_select())) {
+        ORTE_ERROR_LOG(ret);
+        error = "opal_pmix_base_select";
+        goto error;
+    }
+    /* set the event base */
+    opal_pmix_base_set_evbase(orte_event_base);
+
     /* setup the routed info - the selected routed component
-     * will know what to do. 
+     * will know what to do.
      */
     if (ORTE_SUCCESS != (ret = orte_routed.init_routes(ORTE_PROC_MY_NAME->jobid, NULL))) {
         ORTE_ERROR_LOG(ret);
         error = "orte_routed.init_routes";
         goto error;
     }
-    
+
+    /* setup the PMIx server */
+    if (ORTE_SUCCESS != (ret = pmix_server_init())) {
+        ORTE_ERROR_LOG(ret);
+        error = "pmix server init";
+        goto error;
+    }
+
     /* setup I/O forwarding system - must come after we init routes */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_iof_base_framework, 0))) {
         ORTE_ERROR_LOG(ret);
@@ -667,7 +680,7 @@ static int rte_init(void)
         error = "orte_iof_base_select";
         goto error;
     }
-    
+
     /* setup the FileM */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_filem_base_framework, 0))) {
         ORTE_ERROR_LOG(ret);
@@ -680,62 +693,6 @@ static int rte_init(void)
         goto error;
     }
 
-#if OPAL_ENABLE_FT_CR == 1
-    /*
-     * Setup the SnapC
-     */
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_snapc_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_snapc_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_sstore_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sstore_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = orte_snapc_base_select(ORTE_PROC_IS_HNP, !ORTE_PROC_IS_DAEMON))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_snapc_base_select";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = orte_sstore_base_select())) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sstore_base_select";
-        goto error;
-    }
-
-    /* For HNP, ORTE doesn't need the OPAL CR stuff */
-    opal_cr_set_enabled(false);
-#else
-    opal_cr_set_enabled(false);
-#endif
-
-    /*
-     * Initalize the CR setup
-     * Note: Always do this, even in non-FT builds.
-     * If we don't some user level tools may hang.
-     */
-    if (ORTE_SUCCESS != (ret = orte_cr_init())) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_cr_init";
-        goto error;
-    }
-    
-    /* setup the SENSOR framework */
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_sensor_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sensor_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = orte_sensor_base_select())) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sensor_select";
-        goto error;
-    }
-    /* start the local sensors */
-    orte_sensor.start(ORTE_PROC_MY_NAME->jobid);
-    
     /* setup the dfs framework */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_dfs_base_framework, 0))) {
         ORTE_ERROR_LOG(ret);
@@ -748,6 +705,18 @@ static int rte_init(void)
         goto error;
     }
 
+    /* setup the schizo framework */
+    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_schizo_base_framework, 0))) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_schizo_base_open";
+        goto error;
+    }
+    if (ORTE_SUCCESS != (ret = orte_schizo_base_select())) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_schizo_select";
+        goto error;
+    }
+
     /* if a tool has launched us and is requesting event reports,
      * then set its contact info into the comm system
      */
@@ -764,7 +733,7 @@ static int rte_init(void)
        it should; but when it *is* doing something, we do not want it
        to be unnecessarily delayed because it voluntarily yielded the
        processor in the middle of its work.
-     
+
        For example: when a message arrives at orterun, we want the
        OS to wake us up in a timely fashion (which most OS's
        seem good about doing) and then we want orterun to process
@@ -786,7 +755,19 @@ static int rte_init(void)
                        "orte_init:startup:internal-failure",
                        true, error, ORTE_ERROR_NAME(ret), ret);
     }
-    
+
+    /* remove my contact info file, if we have session directories */
+    if (NULL != orte_process_info.job_session_dir) {
+        jobfam_dir = opal_dirname(orte_process_info.job_session_dir);
+        contact_path = opal_os_path(false, jobfam_dir, "contact.txt", NULL);
+        free(jobfam_dir);
+        unlink(contact_path);
+        free(contact_path);
+    }
+    /* remove our use of the session directory tree */
+    orte_session_dir_finalize(ORTE_PROC_MY_NAME);
+    /* ensure we scrub the session directory tree */
+    orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
     return ORTE_ERR_SILENT;
 }
 
@@ -794,6 +775,8 @@ static int rte_finalize(void)
 {
     char *contact_path;
     char *jobfam_dir;
+    ess_hnp_signal_t *sig;
+    unsigned int i;
 
     if (signals_set) {
         /* Remove the epipe handler */
@@ -801,31 +784,34 @@ static int rte_finalize(void)
         /* remove the term handler */
         opal_event_del(&term_handler);
         /** Remove the USR signal handlers */
-        opal_event_signal_del(&sigusr1_handler);
-        opal_event_signal_del(&sigusr2_handler);
-        if (orte_forward_job_control) {
-            opal_event_signal_del(&sigtstp_handler);
-            opal_event_signal_del(&sigcont_handler);
+        i = 0;
+        OPAL_LIST_FOREACH(sig, &mca_ess_hnp_component.signals, ess_hnp_signal_t) {
+            opal_event_signal_del(forward_signals_events + i);
+            ++i;
         }
+        free (forward_signals_events);
+        forward_signals_events = NULL;
         signals_set = false;
     }
 
-    /* stop the local sensors */
-    orte_sensor.stop(ORTE_PROC_MY_NAME->jobid);
-    (void) mca_base_framework_close(&orte_sensor_base_framework);
+    /* shutdown the pmix server */
+    pmix_server_finalize();
+    (void) mca_base_framework_close(&opal_pmix_base_framework);
+    /* cleanup our data server */
+    orte_data_server_finalize();
 
-    /* close the dfs */
+    (void) mca_base_framework_close(&orte_schizo_base_framework);
     (void) mca_base_framework_close(&orte_dfs_base_framework);
     (void) mca_base_framework_close(&orte_filem_base_framework);
     /* output any lingering stdout/err data */
     fflush(stdout);
     fflush(stderr);
     (void) mca_base_framework_close(&orte_iof_base_framework);
+    (void) mca_base_framework_close(&orte_rtc_base_framework);
     (void) mca_base_framework_close(&orte_odls_base_framework);
     (void) mca_base_framework_close(&orte_rmaps_base_framework);
     (void) mca_base_framework_close(&orte_ras_base_framework);
     (void) mca_base_framework_close(&orte_grpcomm_base_framework);
-    (void) mca_base_framework_close(&opal_db_base_framework);
     (void) mca_base_framework_close(&orte_routed_base_framework);
     (void) mca_base_framework_close(&orte_plm_base_framework);
     (void) mca_base_framework_close(&orte_errmgr_base_framework);
@@ -847,9 +833,11 @@ static int rte_finalize(void)
     (void) mca_base_framework_close(&orte_rml_base_framework);
     (void) mca_base_framework_close(&orte_oob_base_framework);
 
+    /* remove our use of the session directory tree */
+    orte_session_dir_finalize(ORTE_PROC_MY_NAME);
     /* ensure we scrub the session directory tree */
     orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
-    
+
     /* close the xml output file, if open */
     if (orte_xml_output) {
         fprintf(orte_xml_fp, "</mpirun>\n");
@@ -866,24 +854,21 @@ static void rte_abort(int status, bool report)
 {
     /* do NOT do a normal finalize as this will very likely
      * hang the process. We are aborting due to an abnormal condition
-     * that precludes normal cleanup 
+     * that precludes normal cleanup
      *
-     * We do need to do the following bits to make sure we leave a 
+     * We do need to do the following bits to make sure we leave a
      * clean environment. Taken from orte_finalize():
      * - Assume errmgr cleans up child processes before we exit.
      */
-    
-    /* CRS cleanup since it may have a named pipe and thread active */
-    orte_cr_finalize();
-    
+
     /* ensure we scrub the session directory tree */
     orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
-    
-    /* - Clean out the global structures 
+
+    /* - Clean out the global structures
      * (not really necessary, but good practice)
      */
     orte_proc_info_finalize();
-    
+
     /* just exit */
     exit(status);
 }
@@ -897,13 +882,13 @@ static void clean_abort(int fd, short flags, void *arg)
         if (forcibly_die) {
             /* kill any local procs */
             orte_odls.kill_local_procs(NULL);
-            
+
             /* whack any lingering session directory files from our jobs */
             orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
-            
+
             /* cleanup our data server */
             orte_data_server_finalize();
-            
+
             /* exit with a non-zero status */
             exit(ORTE_ERROR_DEFAULT_EXIT_CODE);
         }
@@ -913,6 +898,8 @@ static void clean_abort(int fd, short flags, void *arg)
         opal_event_add(&term_handler, NULL);
         return;
     }
+    /* ensure we exit with a non-zero status */
+    ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
 
     /* ensure that the forwarding of stdin stops */
     orte_job_term_ordered = true;
@@ -921,7 +908,7 @@ static void clean_abort(int fd, short flags, void *arg)
      * so need to tell them that!
      */
     orte_execute_quiet = true;
-    
+
     if (!orte_never_launched) {
         /* cleanup our data server */
         orte_data_server_finalize();
