@@ -1,12 +1,12 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2012-2013 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2012-2015 Los Alamos National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2013-2014 Intel, Inc.  All rights reserved. 
- * Copyright (c) 2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013-2016 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -15,7 +15,7 @@
 #include "orte/constants.h"
 
 #include "opal/util/output.h"
-#include "opal/mca/db/db.h"
+#include "opal/mca/pmix/pmix.h"
 #include "opal/util/argv.h"
 
 #include "orte/mca/errmgr/errmgr.h"
@@ -28,7 +28,7 @@ static void process_uri(char *uri);
 
 void orte_oob_base_send_nb(int fd, short args, void *cbdata)
 {
-    orte_oob_send_t *cd = (orte_oob_send_t*)cbdata; 
+    orte_oob_send_t *cd = (orte_oob_send_t*)cbdata;
     orte_rml_send_t *msg = cd->msg;
     mca_base_component_list_item_t *cli;
     orte_oob_base_peer_t *pr;
@@ -37,7 +37,10 @@ void orte_oob_base_send_nb(int fd, short args, void *cbdata)
     bool msg_sent;
     mca_oob_base_component_t *component;
     bool reachable;
-    char *rmluri;
+    char *uri;
+
+    /* done with this. release it now */
+    OBJ_RELEASE(cd);
 
     opal_output_verbose(5, orte_oob_base_framework.framework_output,
                         "%s oob:base:send to target %s",
@@ -55,20 +58,27 @@ void orte_oob_base_send_nb(int fd, short args, void *cbdata)
                             ORTE_NAME_PRINT(&msg->dst));
         /* for direct launched procs, the URI might be in the database,
          * so check there next - if it is, the peer object will be added
-         * to our hash table
+         * to our hash table. However, we don't want to chase up to the
+         * server after it, so indicate it is optional
          */
-	if (OPAL_SUCCESS == opal_db.fetch_pointer((opal_identifier_t*)&msg->dst, ORTE_DB_RMLURI,
-                                                  (void **)&rmluri, OPAL_STRING)) {
-            process_uri(rmluri);
-            free(rmluri);
-            if (OPAL_SUCCESS != opal_hash_table_get_value_uint64(&orte_oob_base.peers,
-                                                                 ui64, (void**)&pr) ||
-                NULL == pr) {
-                /* that is just plain wrong */
+        OPAL_MODEX_RECV_VALUE_OPTIONAL(rc, OPAL_PMIX_PROC_URI, &msg->dst,
+                                      (char**)&uri, OPAL_STRING);
+        if (OPAL_SUCCESS == rc ) {
+            if (NULL != uri) {
+                process_uri(uri);
+                if (OPAL_SUCCESS != opal_hash_table_get_value_uint64(&orte_oob_base.peers,
+                                                                     ui64, (void**)&pr) ||
+                    NULL == pr) {
+                    /* that is just plain wrong */
+                    ORTE_ERROR_LOG(ORTE_ERR_ADDRESSEE_UNKNOWN);
+                    msg->status = ORTE_ERR_ADDRESSEE_UNKNOWN;
+                    ORTE_RML_SEND_COMPLETE(msg);
+                    return;
+                }
+            } else {
                 ORTE_ERROR_LOG(ORTE_ERR_ADDRESSEE_UNKNOWN);
                 msg->status = ORTE_ERR_ADDRESSEE_UNKNOWN;
                 ORTE_RML_SEND_COMPLETE(msg);
-                OBJ_RELEASE(cd);
                 return;
             }
         } else {
@@ -91,7 +101,6 @@ void orte_oob_base_send_nb(int fd, short args, void *cbdata)
                                 ORTE_ERROR_LOG(rc);
                                 msg->status = ORTE_ERR_ADDRESSEE_UNKNOWN;
                                 ORTE_RML_SEND_COMPLETE(msg);
-                                OBJ_RELEASE(cd);
                                 return;
                             }
                         }
@@ -104,9 +113,22 @@ void orte_oob_base_send_nb(int fd, short args, void *cbdata)
             }
             /* if nobody could reach it, then that's an error */
             if (!reachable) {
+                /* if we are a daemon or HNP, then it could be that
+                 * this is a local proc we just haven't heard from
+                 * yet due to a race condition. Check that situation */
+                if (ORTE_PROC_IS_DAEMON || ORTE_PROC_IS_HNP) {
+                    ++msg->retries;
+                    if (msg->retries < orte_rml_base.max_retries) {
+                        ORTE_OOB_SEND(msg);
+                        return;
+                    }
+                }
+                opal_output_verbose(5, orte_oob_base_framework.framework_output,
+                                    "%s CANNOT SEND TO %s: TAG %d",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                    ORTE_NAME_PRINT(&msg->dst), msg->tag);
                 msg->status = ORTE_ERR_ADDRESSEE_UNKNOWN;
                 ORTE_RML_SEND_COMPLETE(msg);
-                OBJ_RELEASE(cd);
                 return;
             }
         }
@@ -123,7 +145,6 @@ void orte_oob_base_send_nb(int fd, short args, void *cbdata)
                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                             ORTE_NAME_PRINT(&msg->dst));
         if (ORTE_SUCCESS == (rc = pr->component->send_nb(msg))) {
-            OBJ_RELEASE(cd);
             return;
         }
     }
@@ -136,8 +157,8 @@ void orte_oob_base_send_nb(int fd, short args, void *cbdata)
     msg_sent = false;
     OPAL_LIST_FOREACH(cli, &orte_oob_base.actives, mca_base_component_list_item_t) {
         component = (mca_oob_base_component_t*)cli->cli_component;
-        /* is this peer addressable by this component? */
-        if (!opal_bitmap_is_set_bit(&pr->addressable, component->idx)) {
+        /* is this peer reachable via this component? */
+        if (!component->is_reachable(&msg->dst)) {
             continue;
         }
         /* it is addressable, so attempt to send via that transport */
@@ -154,7 +175,6 @@ void orte_oob_base_send_nb(int fd, short args, void *cbdata)
             ORTE_ERROR_LOG(rc);
             msg->status = rc;
             ORTE_RML_SEND_COMPLETE(msg);
-            OBJ_RELEASE(cd);
             return;
         }
     }
@@ -170,7 +190,6 @@ void orte_oob_base_send_nb(int fd, short args, void *cbdata)
         msg->status = ORTE_ERR_NO_PATH_TO_TARGET;
         ORTE_RML_SEND_COMPLETE(msg);
     }
-    OBJ_RELEASE(cd);
 }
 
 /**
@@ -259,19 +278,19 @@ void orte_oob_base_get_addr(char **uri)
  * info from it if it can. An error is to be returned if NO component
  * can successfully extract a contact.
  */
-static void req_cons(mca_oob_uri_req_t *ptr) 
-{ 
-    ptr->uri = NULL; 
-} 
-static void req_des(mca_oob_uri_req_t *ptr) 
-{ 
-    if (NULL != ptr->uri) { 
-        free(ptr->uri); 
-    } 
-} 
-OBJ_CLASS_INSTANCE(mca_oob_uri_req_t, 
-                   opal_object_t, 
-                   req_cons, req_des); 
+static void req_cons(mca_oob_uri_req_t *ptr)
+{
+    ptr->uri = NULL;
+}
+static void req_des(mca_oob_uri_req_t *ptr)
+{
+    if (NULL != ptr->uri) {
+        free(ptr->uri);
+    }
+}
+OBJ_CLASS_INSTANCE(mca_oob_uri_req_t,
+                   opal_object_t,
+                   req_cons, req_des);
 
 void orte_oob_base_set_addr(int fd, short args, void *cbdata)
 {
@@ -374,31 +393,13 @@ static void process_uri(char *uri)
                                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                     ORTE_NAME_PRINT(&peer), component->oob_base.mca_component_name);
                 opal_bitmap_set_bit(&pr->addressable, component->idx);
+            } else {
+                opal_output_verbose(5, orte_oob_base_framework.framework_output,
+                                    "%s: peer %s is NOT reachable via component %s",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                    ORTE_NAME_PRINT(&peer), component->oob_base.mca_component_name);
             }
         }
     }
     opal_argv_free(uris);
 }
-
-#if 0
-
-int mca_oob_base_ft_event(int state)
-{
-    int rc;
-    mca_oob_module_t *mod;
-
-    /* loop across all available modules in priority order
-     * and call each one's ft_event handler
-     */
-    OPAL_LIST_FOREACH(mod, &mca_oob_base_modules, mca_oob_module_t) {
-        if (NULL != mod->ft_event) {
-            if (ORTE_SUCCESS != (rc = mod->ft_event(state))) {
-                ORTE_ERROR_LOG(rc);
-            }
-        }
-    }
-
-    return ORTE_SUCCESS;
-}
-
-#endif
