@@ -3,27 +3,28 @@
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.  All rights
- *                         reserved. 
- * Copyright (c) 2013 Cisco Systems, Inc.  All rights reserved.
+ *                         reserved.
+ * Copyright (c) 2013      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2014-2016 Intel Corporation.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
 #include "orte_config.h"
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 
-#include "opal/mca/mca.h"
+#include "opal/dss/dss.h"
+#include "orte/mca/mca.h"
 #include "opal/mca/base/mca_base_component_repository.h"
 #include "opal/util/output.h"
 
 #include "orte/mca/rml/rml.h"
 #include "orte/mca/state/state.h"
+#include "orte/runtime/orte_wait.h"
 #include "orte/util/name_fns.h"
 
 #include "orte/mca/rml/base/base.h"
@@ -33,8 +34,9 @@
  * component's public mca_base_component_t struct. */
 #include "orte/mca/rml/base/static-components.h"
 
-orte_rml_module_t orte_rml;
-orte_rml_base_t   orte_rml_base;
+orte_rml_module_t orte_rml = {0};
+orte_rml_base_t   orte_rml_base = {{{0}}};
+OPAL_TIMING_DECLARE(tm_rml)
 
 orte_rml_component_t *orte_rml_component = NULL;
 
@@ -45,7 +47,7 @@ static int orte_rml_base_register(mca_base_register_flag_t flags)
 {
     int var_id;
 
-    /* 
+    /*
      * Which RML Wrapper component to use, if any
      *  - NULL or "" = No wrapper
      *  - ow. select that specific wrapper component
@@ -59,17 +61,58 @@ static int orte_rml_base_register(mca_base_register_flag_t flags)
                                    &orte_rml_base_wrapper);
     (void) mca_base_var_register_synonym(var_id, "orte", "rml",NULL,"wrapper", 0);
 
+    orte_rml_base.max_retries = 3;
+    mca_base_var_register("orte", "rml", "base", "max_retries",
+                           "Max #times to retry sending a message",
+                           MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                           OPAL_INFO_LVL_9,
+                           MCA_BASE_VAR_SCOPE_READONLY,
+                           &orte_rml_base.max_retries);
+
+#if OPAL_ENABLE_TIMING
+    orte_rml_base.timing = false;
+    (void) mca_base_var_register ("orte", "rml", "base", "timing",
+                                  "Enable RML timings",
+                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_rml_base.timing);
+#endif
+
     return ORTE_SUCCESS;
 }
 
+static void cleanup(int sd, short args, void *cbdata)
+{
+    volatile bool *active = (volatile bool*)cbdata;
+
+    OPAL_LIST_DESTRUCT(&orte_rml_base.posted_recvs);
+    if (NULL != active) {
+        *active = false;
+    }
+}
+
 static int orte_rml_base_close(void)
 {
-    opal_list_item_t *item;
+    volatile bool active;
 
-    while (NULL != (item = opal_list_remove_first(&orte_rml_base.posted_recvs))) {
-        OBJ_RELEASE(item);
-    }
-    OBJ_DESTRUCT(&orte_rml_base.posted_recvs);
+    /* because the RML posted recvs list is in a separate
+     * async thread for apps, we can't just destruct it here.
+     * Instead, we push it into that event thread and destruct
+     * it there */
+     if (ORTE_PROC_IS_APP) {
+        opal_event_t ev;
+        active = true;
+        opal_event_set(orte_event_base, &ev, -1,
+                       OPAL_EV_WRITE, cleanup, (void*)&active);
+        opal_event_set_priority(&ev, ORTE_ERROR_PRI);
+        opal_event_active(&ev, OPAL_EV_WRITE, 1);
+        ORTE_WAIT_FOR_COMPLETION(active);
+     } else {
+        /* we can call the destruct directly */
+        cleanup(0, 0, NULL);
+     }
+
+    OPAL_TIMING_REPORT(orte_rml_base.timing, &tm_rml);
 
     return mca_base_framework_components_close(&orte_rml_base_framework, NULL);
 }
@@ -79,7 +122,7 @@ static int orte_rml_base_open(mca_base_open_flag_t flags)
     /* Initialize globals */
     OBJ_CONSTRUCT(&orte_rml_base.posted_recvs, opal_list_t);
     OBJ_CONSTRUCT(&orte_rml_base.unmatched_msgs, opal_list_t);
-    
+    OPAL_TIMING_INIT(&tm_rml);
     /* Open up all available components */
     return mca_base_framework_components_open(&orte_rml_base_framework, flags);
 }
@@ -91,7 +134,8 @@ MCA_BASE_FRAMEWORK_DECLARE(orte, rml, "ORTE Run-Time Messaging Layer",
 
 int orte_rml_base_select(void)
 {
-    opal_list_item_t *item;
+    opal_list_item_t *item, *next;
+    mca_base_component_list_item_t *cli;
 
     int selected_priority = -1;
     orte_rml_component_t *selected_component = NULL;
@@ -104,23 +148,18 @@ int orte_rml_base_select(void)
         return ORTE_SUCCESS;
     }
     selected = true;
-    
-    for (item = opal_list_get_first(&orte_rml_base_framework.framework_components);
-         item != opal_list_get_end(&orte_rml_base_framework.framework_components) ;
-         item = opal_list_get_next(item)) {
-        mca_base_component_list_item_t *cli;
+
+    OPAL_LIST_FOREACH(cli, &orte_rml_base_framework.framework_components, mca_base_component_list_item_t ) {
         orte_rml_component_t* component;
- 
-        cli = (mca_base_component_list_item_t *) item;
         component = (orte_rml_component_t *) cli->cli_component;
 
-        opal_output_verbose(10, orte_rml_base_framework.framework_output, 
+        opal_output_verbose(10, orte_rml_base_framework.framework_output,
                             "orte_rml_base_select: initializing %s component %s",
                             component->rml_version.mca_type_name,
                             component->rml_version.mca_component_name);
 
         if (NULL == component->rml_init) {
-            opal_output_verbose(10, orte_rml_base_framework.framework_output, 
+            opal_output_verbose(10, orte_rml_base_framework.framework_output,
                                 "orte_rml_base_select: no init function; ignoring component");
         } else {
             int priority = 0;
@@ -138,7 +177,7 @@ int orte_rml_base_select(void)
             if(NULL != orte_rml_base_wrapper &&
                /* If this is a wrapper component then save it for later */
                RML_SELECT_WRAPPER_PRIORITY >= priority) {
-                if( 0 == strncmp(component->rml_version.mca_component_name, 
+                if( 0 == strncmp(component->rml_version.mca_component_name,
                                  orte_rml_base_wrapper,
                                  strlen(orte_rml_base_wrapper) ) ) {
                     wrapper_component = component;
@@ -156,34 +195,24 @@ int orte_rml_base_select(void)
         }
     }
 
-    /* 
+    /*
      * Unload all components that were not selected
      */
-    item = opal_list_get_first(&orte_rml_base_framework.framework_components);
-    while (item != opal_list_get_end(&orte_rml_base_framework.framework_components)) {
-        opal_list_item_t* next = opal_list_get_next(item);
-        orte_rml_component_t* component;
-        mca_base_component_list_item_t *cli;
-
-        cli = (mca_base_component_list_item_t *) item;
-        component = (orte_rml_component_t *) cli->cli_component;
+    OPAL_LIST_FOREACH_SAFE(item, next, &orte_rml_base_framework.framework_components, opal_list_item_t) {
+        mca_base_component_list_item_t *cli = (mca_base_component_list_item_t *) item;
+        orte_rml_component_t* component = (orte_rml_component_t *) cli->cli_component;
 
         /* Keep it if it is the wrapper component */
-        if (NULL != wrapper_component &&
-            component == wrapper_component) {
-            item = next;
+        if ((component == wrapper_component) || (component == selected_component)) {
             continue;
         }
         /* Not the selected component */
-        if (component != selected_component) {
-            opal_output_verbose(10, orte_rml_base_framework.framework_output,
-                                "orte_rml_base_select: module %s unloaded",
-                                component->rml_version.mca_component_name);
-
-            mca_base_component_repository_release((mca_base_component_t *) component);
-            OBJ_RELEASE(item);
-        }
-        item = next;
+        opal_output_verbose(10, orte_rml_base_framework.framework_output,
+                            "orte_rml_base_select: module %s unloaded",
+                            component->rml_version.mca_component_name);
+        opal_list_remove_item(&orte_rml_base_framework.framework_components, item);
+        mca_base_component_repository_release((mca_base_component_t *) component);
+        OBJ_RELEASE(item);
     }
 
     /* setup reference to selected module */
@@ -192,7 +221,7 @@ int orte_rml_base_select(void)
         orte_rml_component = selected_component;
     }
 
-    /* If a wrapper component was requested then 
+    /* If a wrapper component was requested then
      * Make sure it can switch out the selected module
      */
     if( NULL != wrapper_component) {
@@ -205,7 +234,7 @@ int orte_rml_base_select(void)
         }
         return ORTE_ERROR;
     }
-    
+
     return ORTE_SUCCESS;
 }
 
@@ -238,9 +267,11 @@ void orte_rml_recv_callback(int status, orte_process_name_t* sender,
 /***   RML CLASS INSTANCES   ***/
 static void send_cons(orte_rml_send_t *ptr)
 {
+    ptr->retries = 0;
     ptr->cbdata = NULL;
     ptr->iov = NULL;
     ptr->buffer = NULL;
+    ptr->data = NULL;
 }
 OBJ_CLASS_INSTANCE(orte_rml_send_t,
                    opal_list_item_t,
@@ -257,6 +288,7 @@ OBJ_CLASS_INSTANCE(orte_rml_send_request_t,
 static void recv_cons(orte_rml_recv_t *ptr)
 {
     ptr->iov.iov_base = NULL;
+    ptr->iov.iov_len = 0;
 }
 static void recv_des(orte_rml_recv_t *ptr)
 {
@@ -302,4 +334,3 @@ static void prq_des(orte_rml_recv_request_t *ptr)
 OBJ_CLASS_INSTANCE(orte_rml_recv_request_t,
                    opal_object_t,
                    prq_cons, prq_des);
-
