@@ -9,13 +9,15 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2013 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2007-2013 Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ * Copyright (c) 2007-2017 Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2007-2016 Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * Copyright (c) 2009      Institut National de Recherche en Informatique
  *                         et Automatique. All rights reserved.
- * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved. 
+ * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2013-2015 Intel, Inc. All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -26,9 +28,7 @@
 #include "orte_config.h"
 #include "orte/constants.h"
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 
 #include <stdio.h>
 #include <ctype.h>
@@ -63,7 +63,8 @@
 #include "opal/mca/base/mca_base_var.h"
 #include "opal/util/daemon_init.h"
 #include "opal/dss/dss.h"
-#include "opal/mca/hwloc/hwloc.h"
+#include "opal/mca/hwloc/hwloc-internal.h"
+#include "opal/mca/pmix/pmix.h"
 
 #include "orte/util/show_help.h"
 #include "orte/util/proc_info.h"
@@ -96,8 +97,10 @@
 #include "orte/runtime/orte_globals.h"
 #include "orte/runtime/orte_locks.h"
 #include "orte/runtime/orte_quit.h"
+#include "orte/runtime/orte_wait.h"
 
 #include "orte/orted/orted.h"
+#include "orte/orted/pmix/pmix_server.h"
 
 /*
  * Globals
@@ -119,12 +122,11 @@ static struct {
     char* num_procs;
     int uri_pipe;
     int singleton_died_pipe;
-    int fail;
-    int fail_delay;
     bool abort;
     bool mapreduce;
     bool tree_spawn;
     char *hnp_topo_sig;
+    bool test_suicide;
 } orted_globals;
 
 /*
@@ -140,18 +142,14 @@ opal_cmd_line_init_t orte_cmd_line_opts[] = {
       &orted_spin_flag, OPAL_CMD_LINE_TYPE_BOOL,
       "Have the orted spin until we can connect a debugger to it" },
 
-    { NULL, '\0', NULL, "debug-failure", 1,
-      &orted_globals.fail, OPAL_CMD_LINE_TYPE_INT,
-      "Have the specified orted fail after init for debugging purposes" },
-    
-    { NULL, '\0', NULL, "debug-failure-delay", 1,
-      &orted_globals.fail_delay, OPAL_CMD_LINE_TYPE_INT,
-      "Have the orted specified for failure delay for the provided number of seconds before failing" },
-    
+    { NULL, '\0', NULL, "test-suicide", 1,
+      &orted_globals.test_suicide, OPAL_CMD_LINE_TYPE_BOOL,
+      "Suicide instead of clean abort after delay" },
+
     { "orte_debug", 'd', NULL, "debug", 0,
       NULL, OPAL_CMD_LINE_TYPE_BOOL,
       "Debug the OpenRTE" },
-        
+
     { "orte_daemonize", '\0', NULL, "daemonize", 0,
       &orted_globals.daemonize, OPAL_CMD_LINE_TYPE_BOOL,
       "Daemonize the orted into the background" },
@@ -171,19 +169,15 @@ opal_cmd_line_init_t orte_cmd_line_opts[] = {
     { "orte_hnp_uri", '\0', NULL, "hnp-uri", 1,
       NULL, OPAL_CMD_LINE_TYPE_STRING,
       "URI for the HNP"},
-    
+
     { "orte_parent_uri", '\0', NULL, "parent-uri", 1,
       NULL, OPAL_CMD_LINE_TYPE_STRING,
       "URI for the parent if tree launch is enabled."},
-    
-    { NULL, '\0', NULL, "tree-spawn", 0,
-      &orted_globals.tree_spawn, OPAL_CMD_LINE_TYPE_BOOL,
-      "Tree spawn is underway"},
-    
+
     { NULL, '\0', NULL, "set-sid", 0,
       &orted_globals.set_sid, OPAL_CMD_LINE_TYPE_BOOL,
       "Direct the orted to separate from the current session"},
-    
+
     { "tmpdir_base", '\0', NULL, "tmpdir", 1,
       NULL, OPAL_CMD_LINE_TYPE_STRING,
       "Set the root for the session directory tree" },
@@ -195,11 +189,11 @@ opal_cmd_line_init_t orte_cmd_line_opts[] = {
     { NULL, '\0', NULL, "singleton-died-pipe", 1,
       &orted_globals.singleton_died_pipe, OPAL_CMD_LINE_TYPE_INT,
       "Watch on indicated pipe for singleton termination"},
-    
+
     { "orte_output_filename", '\0', "output-filename", "output-filename", 1,
       NULL, OPAL_CMD_LINE_TYPE_STRING,
       "Redirect output from application processes into filename.rank" },
-    
+
     { "orte_xterm", '\0', "xterm", "xterm", 1,
       NULL, OPAL_CMD_LINE_TYPE_STRING,
       "Create a new xterm window and display output from the specified ranks there" },
@@ -212,15 +206,13 @@ opal_cmd_line_init_t orte_cmd_line_opts[] = {
       NULL, OPAL_CMD_LINE_TYPE_STRING,
       "Regular expression defining nodes in system" },
 
-#if OPAL_HAVE_HWLOC
     { "orte_hetero_nodes", '\0', NULL, "hetero-nodes", 0,
       NULL, OPAL_CMD_LINE_TYPE_BOOL,
       "Nodes in cluster may differ in topology, so send the topology back from each node [Default = false]" },
-    
+
     { NULL, '\0', NULL, "hnp-topo-sig", 1,
       &orted_globals.hnp_topo_sig, OPAL_CMD_LINE_TYPE_STRING,
       "Topology signature of HNP" },
-#endif
 
     { NULL, '\0', "mapreduce", "mapreduce", 0,
       &orted_globals.mapreduce, OPAL_CMD_LINE_TYPE_BOOL,
@@ -238,18 +230,16 @@ int orte_daemon(int argc, char *argv[])
     char *rml_uri;
     int i;
     opal_buffer_t *buffer;
-    char hostname[100];
-#if OPAL_ENABLE_FT_CR == 1
-    char *tmp_env_var = NULL;
-#endif
-    
+
+    char hostname[OPAL_MAXHOSTNAMELEN];
+    char *coprocessors;
+    uint8_t tflag;
+
     /* initialize the globals */
     memset(&orted_globals, 0, sizeof(orted_globals));
     /* initialize the singleton died pipe to an illegal value so we can detect it was set */
     orted_globals.singleton_died_pipe = -1;
-    /* init the failure orted vpid to an invalid value */
-    orted_globals.fail = ORTE_VPID_INVALID;
-    
+
     /* setup to check common command line options that just report and die */
     cmd_line = OBJ_NEW(opal_cmd_line_t);
     if (OPAL_SUCCESS != opal_cmd_line_create(cmd_line, orte_cmd_line_opts)) {
@@ -266,13 +256,13 @@ int orte_daemon(int argc, char *argv[])
         OBJ_RELEASE(cmd_line);
         return ret;
     }
-    
+
     /*
      * Since this process can now handle MCA/GMCA parameters, make sure to
      * process them.
      */
     mca_base_cmd_line_process_args(cmd_line, &environ, &environ);
-    
+
     /* Ensure that enough of OPAL is setup for us to be able to run */
     /*
      * NOTE: (JJH)
@@ -281,7 +271,7 @@ int orte_daemon(int argc, char *argv[])
      *  line could contain MCA parameters that affect the way opal_init_util()
      *  functions. AMCA parameters are one such option normally received on the
      *  command line that affect the way opal_init_util() behaves.
-     *  It is "safe" to call mca_base_cmd_line_process_args() before 
+     *  It is "safe" to call mca_base_cmd_line_process_args() before
      *  opal_init_util() since mca_base_cmd_line_process_args() does *not*
      *  depend upon opal_init_util() functionality.
      */
@@ -297,18 +287,18 @@ int orte_daemon(int argc, char *argv[])
      * orted was executed - e.g., by .csh
      */
     orte_launch_environ = opal_argv_copy(environ);
-    
+
     /* purge any ess flag set in the environ when we were launched */
-    opal_unsetenv("OMPI_MCA_ess", &orte_launch_environ);
-    
+    opal_unsetenv(OPAL_MCA_PREFIX"ess", &orte_launch_environ);
+
     /* if orte_daemon_debug is set, let someone know we are alive right
      * away just in case we have a problem along the way
      */
     if (orted_globals.debug) {
-        gethostname(hostname, 100);
+        gethostname(hostname, sizeof(hostname));
         fprintf(stderr, "Daemon was launched on %s - beginning to initialize\n", hostname);
     }
-    
+
     /* check for help request */
     if (orted_globals.help) {
         char *args = NULL;
@@ -328,18 +318,9 @@ int orte_daemon(int argc, char *argv[])
     i=0;
     while (orted_spin_flag) {
         i++;
-        if (1000 < i) i=0;        
+        if (1000 < i) i=0;
     }
 
-#if OPAL_ENABLE_FT_CR == 1
-    /* Mark as a tool program */
-    (void) mca_base_var_env_name ("opal_cr_is_tool", &tmp_env_var);
-    opal_setenv(tmp_env_var,
-                "1",
-                true, &environ);
-    free(tmp_env_var);
-#endif
-
     /* if mapreduce set, flag it */
     if (orted_globals.mapreduce) {
         orte_map_reduce = true;
@@ -353,7 +334,7 @@ int orte_daemon(int argc, char *argv[])
        orted_globals.daemonize) {
         opal_daemon_init(NULL);
     }
-    
+
     /* Set the flag telling OpenRTE that I am NOT a
      * singleton, but am "infrastructure" - prevents setting
      * up incorrect infrastructure that only a singleton would
@@ -375,7 +356,6 @@ int orte_daemon(int argc, char *argv[])
      */
     opal_finalize_util();
 
-#if OPAL_HAVE_HWLOC
     /* bind ourselves if so directed */
     if (NULL != orte_daemon_cores) {
         char **cores=NULL, tmp[128];
@@ -404,6 +384,9 @@ int orte_daemon(int argc, char *argv[])
                                    true, orte_process_info.nodename,
                                    orte_daemon_cores);
                     ret = ORTE_ERR_NOT_SUPPORTED;
+                    hwloc_bitmap_free(ours);
+                    hwloc_bitmap_free(pucpus);
+                    hwloc_bitmap_free(res);
                     goto DONE;
                 }
                 hwloc_bitmap_and(pucpus, pu->online_cpuset, pu->allowed_cpuset);
@@ -426,26 +409,25 @@ int orte_daemon(int argc, char *argv[])
             opal_argv_free(cores);
         }
     }
-#endif
 
-    if ((int)ORTE_VPID_INVALID != orted_globals.fail) {
+    if ((int)ORTE_VPID_INVALID != orted_debug_failure) {
         orted_globals.abort=false;
         /* some vpid was ordered to fail. The value can be positive
          * or negative, depending upon the desired method for failure,
          * so need to check both here
          */
-        if (0 > orted_globals.fail) {
-            orted_globals.fail = -1*orted_globals.fail;
+        if (0 > orted_debug_failure) {
+            orted_debug_failure = -1*orted_debug_failure;
             orted_globals.abort = true;
         }
         /* are we the specified vpid? */
-        if ((int)ORTE_PROC_MY_NAME->vpid == orted_globals.fail) {
+        if ((int)ORTE_PROC_MY_NAME->vpid == orted_debug_failure) {
             /* if the user specified we delay, then setup a timer
              * and have it kill us
              */
-            if (0 < orted_globals.fail_delay) {
-                ORTE_TIMER_EVENT(orted_globals.fail_delay, 0, shutdown_callback, ORTE_SYS_PRI);
-                
+            if (0 < orted_debug_failure_delay) {
+                ORTE_TIMER_EVENT(orted_debug_failure_delay, 0, shutdown_callback, ORTE_SYS_PRI);
+
             } else {
                 opal_output(0, "%s is executing clean %s", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                             orted_globals.abort ? "abort" : "abnormal termination");
@@ -455,12 +437,12 @@ int orte_daemon(int argc, char *argv[])
                  * the local session_dir tree and exit
                  */
                 orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
-                
+
                 /* if we were ordered to abort, do so */
                 if (orted_globals.abort) {
                     abort();
                 }
-                
+
                 /* otherwise, return with non-zero status */
                 ret = ORTE_ERROR_DEFAULT_EXIT_CODE;
                 goto DONE;
@@ -474,18 +456,18 @@ int orte_daemon(int argc, char *argv[])
     orte_process_info.my_daemon_uri = orte_rml.get_contact_info();
     ORTE_PROC_MY_DAEMON->jobid = ORTE_PROC_MY_NAME->jobid;
     ORTE_PROC_MY_DAEMON->vpid = ORTE_PROC_MY_NAME->vpid;
-    
+
     /* if I am also the hnp, then update that contact info field too */
     if (ORTE_PROC_IS_HNP) {
         orte_process_info.my_hnp_uri = orte_rml.get_contact_info();
         ORTE_PROC_MY_HNP->jobid = ORTE_PROC_MY_NAME->jobid;
         ORTE_PROC_MY_HNP->vpid = ORTE_PROC_MY_NAME->vpid;
     }
-    
+
     /* setup the primary daemon command receive function */
     orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD, ORTE_RML_TAG_DAEMON,
                             ORTE_RML_PERSISTENT, orte_daemon_recv, NULL);
-    
+
     /* output a message indicating we are alive, our name, and our pid
      * for debugging purposes
      */
@@ -533,11 +515,12 @@ int orte_daemon(int argc, char *argv[])
         orte_app_context_t *app;
         char *tmp, *nptr, *sysinfo;
         int32_t ljob;
-        orte_grpcomm_collective_t *coll;
-        orte_namelist_t *nm;
+        char **singenv=NULL;
 
         /* setup the singleton's job */
         jdata = OBJ_NEW(orte_job_t);
+        /* default to ompi for now */
+        jdata->personality = strdup("ompi");
         orte_plm_base_create_jobid(jdata);
         ljob = ORTE_LOCAL_JOBID(jdata->jobid);
         opal_pointer_array_set_item(orte_job_data, ljob, jdata);
@@ -554,7 +537,7 @@ int orte_daemon(int argc, char *argv[])
         app->app = strdup("singleton");
         app->num_procs = 1;
         opal_pointer_array_add(jdata->apps, app);
-        
+
         /* setup a proc object for the singleton - since we
          * -must- be the HNP, and therefore we stored our
          * node on the global node pool, and since the singleton
@@ -563,7 +546,8 @@ int orte_daemon(int argc, char *argv[])
         proc = OBJ_NEW(orte_proc_t);
         proc->name.jobid = jdata->jobid;
         proc->name.vpid = 0;
-        proc->alive = true;
+        proc->parent = 0;
+        ORTE_FLAG_SET(proc, ORTE_PROC_FLAG_ALIVE);
         proc->state = ORTE_PROC_STATE_RUNNING;
         proc->app_idx = 0;
         /* obviously, it is on my node */
@@ -572,6 +556,10 @@ int orte_daemon(int argc, char *argv[])
         OBJ_RETAIN(node);  /* keep accounting straight */
         opal_pointer_array_add(jdata->procs, proc);
         jdata->num_procs = 1;
+        /* add the node to the job map */
+        OBJ_RETAIN(node);
+        opal_pointer_array_add(jdata->map->nodes, node);
+        jdata->map->num_nodes++;
         /* and it obviously is on the node */
         OBJ_RETAIN(proc);
         opal_pointer_array_add(node->procs, proc);
@@ -585,53 +573,26 @@ int orte_daemon(int argc, char *argv[])
         proc->node_rank = 0;
         proc->app_rank = 0;
         proc->state = ORTE_PROC_STATE_RUNNING;
-        proc->alive = true;
         proc->app_idx = 0;
-        proc->local_proc = true;
+        ORTE_FLAG_SET(proc, ORTE_PROC_FLAG_LOCAL);
 
-        /* account for the collectives in its modex/barriers */
-        jdata->peer_modex = orte_grpcomm_base_get_coll_id();
-        coll = orte_grpcomm_base_setup_collective(jdata->peer_modex);
-        nm = OBJ_NEW(orte_namelist_t);
-        nm->name.jobid = jdata->jobid;
-        nm->name.vpid = ORTE_VPID_WILDCARD;
-        opal_list_append(&coll->participants, &nm->super);
-
-        jdata->peer_init_barrier = orte_grpcomm_base_get_coll_id();
-        coll = orte_grpcomm_base_setup_collective(jdata->peer_init_barrier);
-        nm = OBJ_NEW(orte_namelist_t);
-        nm->name.jobid = jdata->jobid;
-        nm->name.vpid = ORTE_VPID_WILDCARD;
-        opal_list_append(&coll->participants, &nm->super);
-
-        jdata->peer_fini_barrier = orte_grpcomm_base_get_coll_id();
-        coll = orte_grpcomm_base_setup_collective(jdata->peer_fini_barrier);
-        nm = OBJ_NEW(orte_namelist_t);
-        nm->name.jobid = jdata->jobid;
-        nm->name.vpid = ORTE_VPID_WILDCARD;
-        opal_list_append(&coll->participants, &nm->super);
-
-        /* need to setup a pidmap for it */
-        if (ORTE_SUCCESS != (ret = orte_util_encode_pidmap(&orte_pidmap, false))) {
-            ORTE_ERROR_LOG(ret);
-            goto DONE;
+        /* register the singleton's nspace with our PMIx server */
+        if (ORTE_SUCCESS != (ret = orte_pmix_server_register_nspace(jdata))) {
+          ORTE_ERROR_LOG(ret);
+          goto DONE;
         }
-
-        /* if we don't yet have a daemon map, then we have to generate one
-         * to pass back to it
-         */
-        if (NULL != orte_nidmap.bytes) {
-            free(orte_nidmap.bytes);
+        /* use setup fork to create the envars needed by the singleton */
+        if (OPAL_SUCCESS != (ret = opal_pmix.server_setup_fork(&proc->name, &singenv))) {
+          ORTE_ERROR_LOG(ret);
+          goto DONE;
         }
-        if (ORTE_SUCCESS != (ret = orte_util_encode_nodemap(&orte_nidmap, false))) {
-            ORTE_ERROR_LOG(ret);
-            goto DONE;
-        }
-
-        /* create a string that contains our uri + sysinfo */
+        nptr = opal_argv_join(singenv, ',');
+        opal_argv_free(singenv);
+        /* create a string that contains our uri + sysinfo + PMIx server URI envars */
         orte_util_convert_sysinfo_to_string(&sysinfo, orte_local_cpu_type, orte_local_cpu_model);
-        asprintf(&tmp, "%s[%s]", orte_process_info.my_daemon_uri, sysinfo);
-	free(sysinfo);
+        asprintf(&tmp, "%s[%s]%s", orte_process_info.my_daemon_uri, sysinfo, nptr);
+	      free(sysinfo);
+        free(nptr);
 
         /* pass that info to the singleton */
         write(orted_globals.uri_pipe, tmp, strlen(tmp)+1); /* need to add 1 to get the NULL */
@@ -645,7 +606,7 @@ int orte_daemon(int argc, char *argv[])
          * we may start as the result of a comm_spawn
          */
         for (i=0; NULL != environ[i]; i++) {
-            if (0 == strncmp(environ[i], "OMPI_MCA", 8)) {
+            if (0 == strncmp(environ[i], OPAL_MCA_PREFIX, 9)) {
                 /* make a copy to manipulate */
                 tmp = strdup(environ[i]);
                 /* find the equal sign */
@@ -653,7 +614,7 @@ int orte_daemon(int argc, char *argv[])
                 *nptr = '\0';
                 nptr++;
                 /* add the mca param to the orted cmd line */
-                opal_argv_append_nosize(&orted_cmd_line, "-mca");
+                opal_argv_append_nosize(&orted_cmd_line, "-"OPAL_MCA_CMD_LINE_ID);
                 opal_argv_append_nosize(&orted_cmd_line, &tmp[9]);
                 opal_argv_append_nosize(&orted_cmd_line, nptr);
                 free(tmp);
@@ -754,12 +715,12 @@ int orte_daemon(int argc, char *argv[])
         if (orte_retain_aliases) {
             char **aliases=NULL;
             uint8_t naliases, ni;
-            char hostname[ORTE_MAX_HOSTNAME_SIZE];
+            char hostname[OPAL_MAXHOSTNAMELEN];
 
             /* if we stripped the prefix or removed the fqdn,
              * include full hostname as an alias
              */
-            gethostname(hostname, ORTE_MAX_HOSTNAME_SIZE);
+            gethostname(hostname, sizeof(hostname));
             if (strlen(orte_process_info.nodename) < strlen(hostname)) {
                 opal_argv_append_nosize(&aliases, hostname);
             }
@@ -768,55 +729,53 @@ int orte_daemon(int argc, char *argv[])
             if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &naliases, 1, OPAL_UINT8))) {
                 ORTE_ERROR_LOG(ret);
                 OBJ_RELEASE(buffer);
+                opal_argv_free(aliases);
                 goto DONE;
             }
             for (ni=0; ni < naliases; ni++) {
                 if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &aliases[ni], 1, OPAL_STRING))) {
                     ORTE_ERROR_LOG(ret);
                     OBJ_RELEASE(buffer);
+                    opal_argv_free(aliases);
                     goto DONE;
                 }
             }
             opal_argv_free(aliases);
         }
 
-#if OPAL_HAVE_HWLOC
-        {
-            char *coprocessors;
-            uint8_t tflag;
-                        
-            /* add the local topology, if different from the HNP's or user directed us to,
-             * but always if we are the first daemon to ensure we get a compute node */
-            if (1 == ORTE_PROC_MY_NAME->vpid || orte_hetero_nodes ||
-                0 != strcmp(orte_topo_signature, orted_globals.hnp_topo_sig)) {
-                tflag = 1;
-                if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &tflag, 1, OPAL_UINT8))) {
-                    ORTE_ERROR_LOG(ret);
-                }
-                if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &orte_topo_signature, 1, OPAL_STRING))) {
-                    ORTE_ERROR_LOG(ret);
-                }
-                if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &opal_hwloc_topology, 1, OPAL_HWLOC_TOPO))) {
-                    ORTE_ERROR_LOG(ret);
-                }
-            } else {
-                tflag = 0;
-                if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &tflag, 1, OPAL_UINT8))) {
-                    ORTE_ERROR_LOG(ret);
-                }
-            }
-            /* detect and add any coprocessors */
-            coprocessors = opal_hwloc_base_find_coprocessors(opal_hwloc_topology);
-            if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &coprocessors, 1, OPAL_STRING))) {
+        /* add the local topology, if different from the HNP's or user directed us to,
+         * but always if we are the first daemon to ensure we get a compute node */
+        if (1 == ORTE_PROC_MY_NAME->vpid || orte_hetero_nodes ||
+            0 != strcmp(orte_topo_signature, orted_globals.hnp_topo_sig)) {
+            tflag = 1;
+            if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &tflag, 1, OPAL_UINT8))) {
                 ORTE_ERROR_LOG(ret);
             }
-            /* see if I am on a coprocessor */
-            coprocessors = opal_hwloc_base_check_on_coprocessor();
-            if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &coprocessors, 1, OPAL_STRING))) {
+            if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &orte_topo_signature, 1, OPAL_STRING))) {
+                ORTE_ERROR_LOG(ret);
+            }
+            if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &opal_hwloc_topology, 1, OPAL_HWLOC_TOPO))) {
+                ORTE_ERROR_LOG(ret);
+            }
+        } else {
+            tflag = 0;
+            if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &tflag, 1, OPAL_UINT8))) {
                 ORTE_ERROR_LOG(ret);
             }
         }
-#endif
+        /* detect and add any coprocessors */
+        coprocessors = opal_hwloc_base_find_coprocessors(opal_hwloc_topology);
+        if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &coprocessors, 1, OPAL_STRING))) {
+            ORTE_ERROR_LOG(ret);
+        }
+        /* see if I am on a coprocessor */
+        coprocessors = opal_hwloc_base_check_on_coprocessor();
+        if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &coprocessors, 1, OPAL_STRING))) {
+            ORTE_ERROR_LOG(ret);
+        }
+        if (NULL!= coprocessors) {
+            free(coprocessors);
+        }
 
         /* send to the HNP's callback - will be routed if routes are available */
         if (0 > (ret = orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, buffer,
@@ -841,11 +800,12 @@ int orte_daemon(int argc, char *argv[])
             "orte_ess_vpid",
             "orte_ess_num_procs",
             "orte_parent_uri",
+            "mca_base_env_list",
             NULL
         };
         for (i=0; i < argc; i++) {
-            if (0 == strcmp("-mca",  argv[i]) ||
-                0 == strcmp("--mca", argv[i]) ) {
+            if (0 == strcmp("-"OPAL_MCA_CMD_LINE_ID,  argv[i]) ||
+                0 == strcmp("--"OPAL_MCA_CMD_LINE_ID, argv[i]) ) {
                 ignore = false;
                 /* see if this is something we cannot pass along */
                 for (k=0; NULL != no_keep[k]; k++) {
@@ -877,7 +837,7 @@ int orte_daemon(int argc, char *argv[])
             }
         }
     }
-            
+
     if (orte_debug_daemons_flag) {
         opal_output(0, "%s orted: up and running - waiting for commands!", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
     }
@@ -921,14 +881,18 @@ static void shutdown_callback(int fd, short flags, void *arg)
         /* release the timer */
         OBJ_RELEASE(tm);
     }
-    
+
     /* if we were ordered to abort, do so */
     if (orted_globals.abort) {
-        opal_output(0, "%s is executing clean abort", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+        opal_output(0, "%s is executing %s abort", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                    (orted_globals.test_suicide) ? "suicide" : "clean");
         /* do -not- call finalize as this will send a message to the HNP
          * indicating clean termination! Instead, just kill our
          * local procs, forcibly cleanup the local session_dir tree, and abort
          */
+        if (orted_globals.test_suicide) {
+            exit(1);
+        }
         orte_odls.kill_local_procs(NULL);
         orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
         abort();
