@@ -1,8 +1,11 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2013-2015 Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2014      Research Organization for Information Science
+ * Copyright (c) 2014-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -19,6 +22,7 @@
 
 #include "oshmem_config.h"
 #include "opal/datatype/opal_convertor.h"
+#include "opal/mca/memchecker/base/base.h"
 #include "orte/include/orte/types.h"
 #include "orte/runtime/orte_globals.h"
 #include "oshmem/mca/spml/ikrit/spml_ikrit.h"
@@ -38,6 +42,19 @@
 #define SPML_IKRIT_PUT_DEBUG    0
 #endif
 
+#define SPML_IKRIT_MXM_POST_SEND(sreq) \
+do { \
+    mxm_error_t err; \
+    err = mxm_req_send(&sreq); \
+    if (MXM_OK != err) { \
+        SPML_ERROR("mxm_req_send (op=%d) failed: %s - aborting", \
+                   sreq.opcode, \
+                   mxm_error_string(err)); \
+        oshmem_shmem_abort(-1); \
+        return OSHMEM_ERROR; \
+    } \
+} while(0)
+
 typedef struct spml_ikrit_am_hdr {
     uint64_t va;
 } spml_ikrit_am_hdr_t;
@@ -85,10 +102,16 @@ static inline mxm_mem_key_t *to_mxm_mkey(sshmem_mkey_t *mkey) {
 }
 #endif
 
+
 static inline void mca_spml_irkit_req_wait(mxm_req_base_t *req)
 {
-    while (!mxm_req_test(req))
+    do {
+        /* do at least one progress since
+         * with some TLs (self, shm) request
+         * can be completed immediately
+         */
         opal_progress();
+    } while (!mxm_req_test(req));
 }
 
 static int mca_spml_ikrit_put_request_free(struct oshmem_request_t** request)
@@ -96,11 +119,12 @@ static int mca_spml_ikrit_put_request_free(struct oshmem_request_t** request)
     mca_spml_ikrit_put_request_t *put_req =
             *(mca_spml_ikrit_put_request_t **) request;
 
-    assert(false == put_req->req_put.req_base.req_free_called);
     OPAL_THREAD_LOCK(&oshmem_request_lock);
+    assert(false == put_req->req_put.req_base.req_free_called);
     put_req->req_put.req_base.req_free_called = true;
-    OMPI_FREE_LIST_RETURN_MT( &mca_spml_base_put_requests,
-                          (ompi_free_list_item_t*)put_req);
+    opal_free_list_return (&mca_spml_base_put_requests,
+                           (opal_free_list_item_t*)put_req);
+    opal_memchecker_base_mem_noaccess(put_req, sizeof(*put_req));
     OPAL_THREAD_UNLOCK(&oshmem_request_lock);
 
     *request = SHMEM_REQUEST_NULL; /*MPI_REQUEST_NULL;*/
@@ -144,11 +168,12 @@ static int mca_spml_ikrit_get_request_free(struct oshmem_request_t** request)
     mca_spml_ikrit_get_request_t *get_req =
             *(mca_spml_ikrit_get_request_t **) request;
 
-    assert(false == get_req->req_get.req_base.req_free_called);
     OPAL_THREAD_LOCK(&oshmem_request_lock);
+    assert(false == get_req->req_get.req_base.req_free_called);
     get_req->req_get.req_base.req_free_called = true;
-    OMPI_FREE_LIST_RETURN_MT( &mca_spml_base_get_requests,
-                          (ompi_free_list_item_t*)get_req);
+    opal_free_list_return (&mca_spml_base_get_requests,
+                           (opal_free_list_item_t*)get_req);
+    opal_memchecker_base_mem_noaccess(get_req, sizeof(*get_req));
     OPAL_THREAD_UNLOCK(&oshmem_request_lock);
 
     *request = SHMEM_REQUEST_NULL; /*MPI_REQUEST_NULL;*/
@@ -164,7 +189,7 @@ static int mca_spml_ikrit_get_request_cancel(struct oshmem_request_t * request,
 
 static void mca_spml_ikrit_get_request_construct(mca_spml_ikrit_get_request_t* req)
 {
-    req->req_get.req_base.req_type = MCA_SPML_REQUEST_PUT;
+    req->req_get.req_base.req_type = MCA_SPML_REQUEST_GET;
     req->req_get.req_base.req_oshmem.req_free = mca_spml_ikrit_get_request_free;
     req->req_get.req_base.req_oshmem.req_cancel =
             mca_spml_ikrit_get_request_cancel;
@@ -201,6 +226,9 @@ mca_spml_ikrit_t mca_spml_ikrit = {
         mca_spml_base_wait,
         mca_spml_base_wait_nb,
         mca_spml_ikrit_fence,
+        mca_spml_base_rmkey_unpack,
+        mca_spml_base_rmkey_free,
+
         (void*)&mca_spml_ikrit
     }
 };
@@ -234,11 +262,15 @@ void mca_spml_ikrit_dump_stats()
 static inline mca_spml_ikrit_put_request_t *alloc_put_req(void)
 {
     mca_spml_ikrit_put_request_t *req;
-    ompi_free_list_item_t* item;
+    opal_free_list_item_t* item;
 
-    OMPI_FREE_LIST_WAIT_MT(&mca_spml_base_put_requests, item);
+    item = opal_free_list_wait (&mca_spml_base_put_requests);
 
     req = (mca_spml_ikrit_put_request_t *) item;
+    opal_memchecker_base_mem_undefined(req, sizeof(*req));
+    opal_memchecker_base_mem_defined(&req->req_put.req_base,
+                                     sizeof(req->req_put.req_base));
+
     req->req_put.req_base.req_free_called = false;
     req->req_put.req_base.req_oshmem.req_complete = false;
 
@@ -248,11 +280,15 @@ static inline mca_spml_ikrit_put_request_t *alloc_put_req(void)
 static inline mca_spml_ikrit_get_request_t *alloc_get_req(void)
 {
     mca_spml_ikrit_get_request_t *req;
-    ompi_free_list_item_t* item;
+    opal_free_list_item_t* item;
 
-    OMPI_FREE_LIST_WAIT_MT(&mca_spml_base_get_requests, item);
+    item = opal_free_list_wait (&mca_spml_base_get_requests);
 
     req = (mca_spml_ikrit_get_request_t *) item;
+    opal_memchecker_base_mem_undefined(req, sizeof(*req));
+    opal_memchecker_base_mem_defined(&req->req_get.req_base,
+                                     sizeof(req->req_get.req_base));
+
     req->req_get.req_base.req_free_called = false;
     req->req_get.req_base.req_oshmem.req_complete = false;
 
@@ -266,27 +302,27 @@ int mca_spml_ikrit_enable(bool enable)
         return OSHMEM_SUCCESS;
     }
 
-    ompi_free_list_init_new(&mca_spml_base_put_requests,
-                            sizeof(mca_spml_ikrit_put_request_t),
-                            opal_cache_line_size,
-                            OBJ_CLASS(mca_spml_ikrit_put_request_t),
-                            0,
-                            opal_cache_line_size,
-                            mca_spml_ikrit.free_list_num,
-                            mca_spml_ikrit.free_list_max,
-                            mca_spml_ikrit.free_list_inc,
-                            NULL );
+    opal_free_list_init (&mca_spml_base_put_requests,
+                         sizeof(mca_spml_ikrit_put_request_t),
+                         opal_cache_line_size,
+                         OBJ_CLASS(mca_spml_ikrit_put_request_t),
+                         0,
+                         opal_cache_line_size,
+                         mca_spml_ikrit.free_list_num,
+                         mca_spml_ikrit.free_list_max,
+                         mca_spml_ikrit.free_list_inc,
+                         NULL, 0, NULL, NULL, NULL);
 
-    ompi_free_list_init_new(&mca_spml_base_get_requests,
-                            sizeof(mca_spml_ikrit_get_request_t),
-                            opal_cache_line_size,
-                            OBJ_CLASS(mca_spml_ikrit_get_request_t),
-                            0,
-                            opal_cache_line_size,
-                            mca_spml_ikrit.free_list_num,
-                            mca_spml_ikrit.free_list_max,
-                            mca_spml_ikrit.free_list_inc,
-                            NULL );
+    opal_free_list_init (&mca_spml_base_get_requests,
+                         sizeof(mca_spml_ikrit_get_request_t),
+                         opal_cache_line_size,
+                         OBJ_CLASS(mca_spml_ikrit_get_request_t),
+                         0,
+                         opal_cache_line_size,
+                         mca_spml_ikrit.free_list_num,
+                         mca_spml_ikrit.free_list_max,
+                         mca_spml_ikrit.free_list_inc,
+                         NULL, 0, NULL, NULL, NULL);
 
     mca_spml_ikrit.enabled = true;
 
@@ -295,31 +331,31 @@ int mca_spml_ikrit_enable(bool enable)
 
 static int create_ptl_idx(int dst_pe)
 {
-    oshmem_proc_t *proc;
+    ompi_proc_t *proc;
 
     proc = oshmem_proc_group_find(oshmem_group_all, dst_pe);
 
-    proc->transport_ids = (char *) malloc(MXM_PTL_LAST * sizeof(char));
-    if (!proc->transport_ids)
+    OSHMEM_PROC_DATA(proc)->transport_ids = (char *) malloc(MXM_PTL_LAST * sizeof(char));
+    if (NULL == OSHMEM_PROC_DATA(proc)->transport_ids)
         return OSHMEM_ERROR;
 
-    proc->num_transports = 1;
+    OSHMEM_PROC_DATA(proc)->num_transports = 1;
 #if MXM_API < MXM_VERSION(2,0)
     if (oshmem_my_proc_id() == dst_pe)
-        proc->transport_ids[0] = MXM_PTL_SELF;
+        OSHMEM_PROC_DATA(proc)->transport_ids[0] = MXM_PTL_SELF;
     else
 #endif
-        proc->transport_ids[0] = MXM_PTL_RDMA;
+        OSHMEM_PROC_DATA(proc)->transport_ids[0] = MXM_PTL_RDMA;
     return OSHMEM_SUCCESS;
 }
 
 static void destroy_ptl_idx(int dst_pe)
 {
-    oshmem_proc_t *proc;
+    ompi_proc_t *proc;
 
     proc = oshmem_proc_group_find(oshmem_group_all, dst_pe);
-    if (proc->transport_ids)
-        free(proc->transport_ids);
+    if (NULL != OSHMEM_PROC_DATA(proc)->transport_ids)
+        free(OSHMEM_PROC_DATA(proc)->transport_ids);
 }
 
 static void mxm_peer_construct(mxm_peer_t *p)
@@ -339,7 +375,7 @@ OBJ_CLASS_INSTANCE( mxm_peer_t,
                    mxm_peer_construct,
                    mxm_peer_destruct);
 
-int mca_spml_ikrit_del_procs(oshmem_proc_t** procs, size_t nprocs)
+int mca_spml_ikrit_del_procs(ompi_proc_t** procs, size_t nprocs)
 {
     size_t i, n;
     int my_rank = oshmem_my_proc_id();
@@ -357,28 +393,24 @@ int mca_spml_ikrit_del_procs(oshmem_proc_t** procs, size_t nprocs)
 
     for (n = 0; n < nprocs; n++) {
         i = (my_rank + n) % nprocs;
-        if (mca_spml_ikrit.mxm_peers[i]->mxm_conn) {
-            mxm_ep_disconnect(mca_spml_ikrit.mxm_peers[i]->mxm_conn);
-        }
-        if (mca_spml_ikrit.hw_rdma_channel && mca_spml_ikrit.mxm_peers[i]->mxm_hw_rdma_conn) {
+        mxm_ep_disconnect(mca_spml_ikrit.mxm_peers[i]->mxm_conn);
+        if (mca_spml_ikrit.hw_rdma_channel) {
+            assert(mca_spml_ikrit.mxm_peers[i]->mxm_hw_rdma_conn != mca_spml_ikrit.mxm_peers[i]->mxm_conn);
             mxm_ep_disconnect(mca_spml_ikrit.mxm_peers[i]->mxm_hw_rdma_conn);
         }
         destroy_ptl_idx(i);
-        if (mca_spml_ikrit.mxm_peers[i]) {
-            OBJ_RELEASE(mca_spml_ikrit.mxm_peers[i]);
-        }
+        OBJ_RELEASE(mca_spml_ikrit.mxm_peers[i]);
     }
-    if (mca_spml_ikrit.mxm_peers)
-        free(mca_spml_ikrit.mxm_peers);
+    free(mca_spml_ikrit.mxm_peers);
 
     return OSHMEM_SUCCESS;
 }
 
-int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
+int mca_spml_ikrit_add_procs(ompi_proc_t** procs, size_t nprocs)
 {
     spml_ikrit_mxm_ep_conn_info_t *ep_info = NULL;
     spml_ikrit_mxm_ep_conn_info_t *ep_hw_rdma_info = NULL;
-    spml_ikrit_mxm_ep_conn_info_t my_ep_info;
+    spml_ikrit_mxm_ep_conn_info_t my_ep_info = {0,};
 #if MXM_API < MXM_VERSION(2,0)
     mxm_conn_req_t *conn_reqs;
     int timeout;
@@ -388,7 +420,7 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
     mxm_error_t err;
     size_t i, n;
     int rc = OSHMEM_ERROR;
-    oshmem_proc_t *proc_self;
+    ompi_proc_t *proc_self;
     int my_rank = oshmem_my_proc_id();
 
     OBJ_CONSTRUCT(&mca_spml_ikrit.active_peers, opal_list_t);
@@ -401,20 +433,18 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
     }
     memset(conn_reqs, 0x0, sizeof(mxm_conn_req_t));
 #endif
-    ep_info = malloc(nprocs * sizeof(spml_ikrit_mxm_ep_conn_info_t));
+    ep_info = calloc(sizeof(spml_ikrit_mxm_ep_conn_info_t), nprocs);
     if (NULL == ep_info) {
         rc = OSHMEM_ERR_OUT_OF_RESOURCE;
         goto bail;
     }
-    memset(ep_info, 0x0, sizeof(spml_ikrit_mxm_ep_conn_info_t));
 
     if (mca_spml_ikrit.hw_rdma_channel) {
-        ep_hw_rdma_info = malloc(nprocs * sizeof(spml_ikrit_mxm_ep_conn_info_t));
+        ep_hw_rdma_info = calloc(sizeof(spml_ikrit_mxm_ep_conn_info_t), nprocs);
         if (NULL == ep_hw_rdma_info) {
             rc = OSHMEM_ERR_OUT_OF_RESOURCE;
             goto bail;
         }
-        memset(ep_hw_rdma_info, 0x0, sizeof(spml_ikrit_mxm_ep_conn_info_t));
     }
 
     mca_spml_ikrit.mxm_peers = (mxm_peer_t **) malloc(nprocs
@@ -523,16 +553,21 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
     /* Save returned connections */
     for (i = 0; i < nprocs; ++i) {
         mca_spml_ikrit.mxm_peers[i]->mxm_conn = conn_reqs[i].conn;
-        if (OSHMEM_SUCCESS != create_ptl_idx(i))
+        if (OSHMEM_SUCCESS != create_ptl_idx(i)) {
+            rc = OSHMEM_ERR_CONNECTION_FAILED;
             goto bail;
+        }
 
         mxm_conn_ctx_set(conn_reqs[i].conn, mca_spml_ikrit.mxm_peers[i]);
     }
 
-    free(conn_reqs);
+    if (conn_reqs)
+        free(conn_reqs);
 #endif
-    free(ep_info);
-    free(ep_hw_rdma_info);
+    if (ep_info)
+        free(ep_info);
+    if (ep_hw_rdma_info)
+        free(ep_hw_rdma_info);
 
 #if MXM_API >= MXM_VERSION(2,0)
     if (mca_spml_ikrit.bulk_connect) {
@@ -545,17 +580,17 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
     proc_self = oshmem_proc_group_find(oshmem_group_all, my_rank);
     /* identify local processes and change transport to SHM */
     for (i = 0; i < nprocs; i++) {
-        if (procs[i]->proc_name.jobid != proc_self->proc_name.jobid||
-        !OPAL_PROC_ON_LOCAL_NODE(procs[i]->proc_flags)) {
+        if (procs[i]->super.proc_name.jobid != proc_self->super.proc_name.jobid ||
+            !OPAL_PROC_ON_LOCAL_NODE(procs[i]->super.proc_flags)) {
             continue;
         }
         if (procs[i] == proc_self)
-        continue;
+            continue;
 
         /* use zcopy for put/get via sysv shared memory */
-        procs[i]->transport_ids[0] = MXM_PTL_SHM;
-        procs[i]->transport_ids[1] = MXM_PTL_RDMA;
-        procs[i]->num_transports = 2;
+        OSHMEM_PROC_DATA(procs[i])->transport_ids[0] = MXM_PTL_SHM;
+        OSHMEM_PROC_DATA(procs[i])->transport_ids[1] = MXM_PTL_RDMA;
+        OSHMEM_PROC_DATA(procs[i])->num_transports = 2;
     }
 
     SPML_VERBOSE(50, "*** ADDED PROCS ***");
@@ -563,10 +598,13 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
 
 bail:
 #if MXM_API < MXM_VERSION(2,0)
-    free(conn_reqs);
+	if (conn_reqs)
+		free(conn_reqs);
 #endif
-    free(ep_info);
-    free(ep_hw_rdma_info);
+	if (ep_info)
+		free(ep_info);
+	if (ep_hw_rdma_info)
+		free(ep_hw_rdma_info);
     SPML_ERROR("add procs FAILED rc=%d", rc);
 
     return rc;
@@ -617,7 +655,7 @@ sshmem_mkey_t *mca_spml_ikrit_register(void* addr,
 #if MXM_API < MXM_VERSION(2,0)
             mkeys[i].len = 0;
 #else
-            if (mca_spml_ikrit.ud_only && !mca_spml_ikrit.hw_rdma_channel) {
+            if (mca_spml_ikrit.ud_only) {
                 mkeys[i].len = 0;
                 break;
             }
@@ -644,10 +682,14 @@ sshmem_mkey_t *mca_spml_ikrit_register(void* addr,
             }
 #endif
             break;
+
+        default:
+            SPML_ERROR("unsupported PTL: %d", i);
+            goto error_out;
         }
         SPML_VERBOSE(5,
                      "rank %d ptl %d addr %p size %llu %s",
-                     oshmem_proc_local_proc->proc_name.vpid, i, addr, (unsigned long long)size,
+		     oshmem_proc_pe(oshmem_proc_local()), i, addr, (unsigned long long)size,
                      mca_spml_base_mkey2str(&mkeys[i]));
 
     }
@@ -658,7 +700,7 @@ sshmem_mkey_t *mca_spml_ikrit_register(void* addr,
 error_out:
     mca_spml_ikrit_deregister(mkeys);
 
-    return NULL ;
+    return NULL;
 }
 
 int mca_spml_ikrit_deregister(sshmem_mkey_t *mkeys)
@@ -700,7 +742,7 @@ int mca_spml_ikrit_deregister(sshmem_mkey_t *mkeys)
 
 static inline int get_ptl_id(int dst)
 {
-    oshmem_proc_t *proc;
+    ompi_proc_t *proc;
 
     /* get endpoint and btl */
     proc = oshmem_proc_group_all(dst);
@@ -709,7 +751,7 @@ static inline int get_ptl_id(int dst)
         oshmem_shmem_abort(-1);
         return -1;
     }
-    return proc->transport_ids[0];
+    return OSHMEM_PROC_DATA(proc)->transport_ids[0];
 }
 
 int mca_spml_ikrit_oob_get_mkeys(int pe, uint32_t seg, sshmem_mkey_t *mkeys)
@@ -737,7 +779,7 @@ int mca_spml_ikrit_oob_get_mkeys(int pe, uint32_t seg, sshmem_mkey_t *mkeys)
         mkeys[ptl].len = 0;
         mkeys[ptl].u.key = MAP_SEGMENT_SHM_INVALID;
         return OSHMEM_SUCCESS;
-    } 
+    }
 
     return OSHMEM_ERROR;
 #endif
@@ -763,10 +805,7 @@ static int mca_spml_ikrit_get_helper(mxm_send_req_t *sreq,
     /**
      * Get the address to the remote rkey.
      **/
-    r_mkey = mca_memheap.memheap_get_cached_mkey(src,
-                                                 src_addr,
-                                                 ptl_id,
-                                                 &rva);
+    r_mkey = mca_memheap_base_get_cached_mkey(src, src_addr, ptl_id, &rva);
     if (!r_mkey) {
         SPML_ERROR("pe=%d: %p is not address of shared variable",
                    src, src_addr);
@@ -813,10 +852,7 @@ static inline int mca_spml_ikrit_get_shm(void *src_addr,
     if (ptl_id != MXM_PTL_SHM)
         return OSHMEM_ERROR;
 
-    r_mkey = mca_memheap.memheap_get_cached_mkey(src,
-                                                 src_addr,
-                                                 ptl_id,
-                                                 &rva);
+    r_mkey = mca_memheap_base_get_cached_mkey(src, src_addr, ptl_id, &rva);
     if (!r_mkey) {
         SPML_ERROR("pe=%d: %p is not address of shared variable",
                    src, src_addr);
@@ -838,6 +874,7 @@ static inline int mca_spml_ikrit_get_shm(void *src_addr,
 int mca_spml_ikrit_get(void *src_addr, size_t size, void *dst_addr, int src)
 {
     mxm_send_req_t sreq;
+    mxm_error_t err;
 
     if (0 >= size) {
         return OSHMEM_SUCCESS;
@@ -863,10 +900,9 @@ int mca_spml_ikrit_get(void *src_addr, size_t size, void *dst_addr, int src)
 #endif
     sreq.base.completed_cb = NULL;
 
-    mxm_req_send(&sreq);
-    opal_progress();
-    mca_spml_irkit_req_wait(&sreq.base);
+    SPML_IKRIT_MXM_POST_SEND(sreq);
 
+    mca_spml_irkit_req_wait(&sreq.base);
     if (MXM_OK != sreq.base.error) {
         SPML_ERROR("get request failed: %s - aborting",
                    mxm_error_string(sreq.base.error));
@@ -925,14 +961,8 @@ int mca_spml_ikrit_get_async(void *src_addr,
     get_req->mxm_req.base.context = get_req;
     OPAL_THREAD_ADD32(&mca_spml_ikrit.n_active_gets, 1);
 
-    mxm_req_send(&get_req->mxm_req);
+    SPML_IKRIT_MXM_POST_SEND(get_req->mxm_req);
 
-    if (MXM_OK != get_req->mxm_req.base.error) {
-        SPML_ERROR("get request failed: %s - aborting",
-                   mxm_error_string(get_req->mxm_req.base.error));
-        oshmem_shmem_abort(-1);
-        return OSHMEM_ERROR;
-    }
     return OSHMEM_SUCCESS;
 }
 
@@ -979,7 +1009,7 @@ static int mca_spml_ikrit_mxm_fence(int dst)
     fence_req->mxm_req.base.context = fence_req;
     OPAL_THREAD_ADD32(&mca_spml_ikrit.n_mxm_fences, 1);
 
-    mxm_req_send(&fence_req->mxm_req);
+    SPML_IKRIT_MXM_POST_SEND(fence_req->mxm_req);
     return OSHMEM_SUCCESS;
 }
 
@@ -1051,10 +1081,7 @@ static inline int mca_spml_ikrit_put_internal(void* dst_addr,
 
     ptl_id = get_ptl_id(dst);
     /* Get rkey of remote PE (dst proc) which must be on memheap  */
-    r_mkey = mca_memheap.memheap_get_cached_mkey(dst,
-                                                 dst_addr,
-                                                 ptl_id,
-                                                 &rva);
+    r_mkey = mca_memheap_base_get_cached_mkey(dst, dst_addr, ptl_id, &rva);
     if (!r_mkey) {
         SPML_ERROR("pe=%d: %p is not address of shared variable",
                    dst, dst_addr);
@@ -1078,10 +1105,7 @@ static inline int mca_spml_ikrit_put_internal(void* dst_addr,
         }
         /* segment not mapped - fallback to rmda */
         ptl_id = MXM_PTL_RDMA;
-        r_mkey = mca_memheap.memheap_get_cached_mkey(dst,
-                                                     dst_addr,
-                                                     ptl_id,
-                                                     &rva);
+        r_mkey = mca_memheap_base_get_cached_mkey(dst, dst_addr, ptl_id, &rva);
         if (!r_mkey) {
             SPML_ERROR("pe=%d: %p is not address of shared variable",
                        dst, dst_addr);
@@ -1158,15 +1182,8 @@ static inline int mca_spml_ikrit_put_internal(void* dst_addr,
 
     mca_spml_ikrit.mxm_peers[dst]->n_active_puts++;
 
-    mxm_req_send(&put_req->mxm_req);
+    SPML_IKRIT_MXM_POST_SEND(put_req->mxm_req);
 
-    if (MXM_OK != put_req->mxm_req.base.error) {
-        OPAL_THREAD_ADD32(&mca_spml_ikrit.n_active_puts, -1);
-        SPML_ERROR("put request %p failed: %s - aborting",
-                   (void*)put_req, mxm_error_string(put_req->mxm_req.base.error));
-        oshmem_shmem_abort(-1);
-        return OSHMEM_ERROR;
-    }
     if (need_progress)
         mxm_progress(mca_spml_ikrit.mxm_context);
 
@@ -1177,7 +1194,7 @@ static inline int mca_spml_ikrit_put_internal(void* dst_addr,
  * Problems:
  * - slighly worse performance than impl based on non buffered put
  * - fence complexity is O(n_active_connections) instead of O(n_connections_with_outstanding_puts).
- *   Later is bounded by the network RTT & mxm ack timer. 
+ *   Later is bounded by the network RTT & mxm ack timer.
  */
 int mca_spml_ikrit_put_simple(void* dst_addr,
                               size_t size,
@@ -1193,11 +1210,7 @@ int mca_spml_ikrit_put_simple(void* dst_addr,
 
     ptl_id = get_ptl_id(dst);
     /* Get rkey of remote PE (dst proc) which must be on memheap  */
-    r_mkey = mca_memheap.memheap_get_cached_mkey(dst,
-                                                 //(unsigned long) dst_addr,
-                                                 dst_addr,
-                                                 ptl_id,
-                                                 &rva);
+    r_mkey = mca_memheap_base_get_cached_mkey(dst, dst_addr, ptl_id, &rva);
     if (!r_mkey) {
         SPML_ERROR("pe=%d: %p is not address of shared variable",
                    dst, dst_addr);
@@ -1220,7 +1233,7 @@ int mca_spml_ikrit_put_simple(void* dst_addr,
         }
         /* segment not mapped - fallback to rmda */
         ptl_id = MXM_PTL_RDMA;
-        r_mkey = mca_memheap.memheap_get_cached_mkey(dst,
+        r_mkey = mca_memheap_base_get_cached_mkey(dst,
                                                      //(unsigned long) dst_addr,
                                                      dst_addr,
                                                      ptl_id,
@@ -1269,13 +1282,7 @@ int mca_spml_ikrit_put_simple(void* dst_addr,
         mca_spml_ikrit.mxm_peers[dst]->need_fence = 1;
     }
 
-    mxm_req_send(&mxm_req);
-    if (MXM_OK != mxm_req.base.error) {
-        SPML_ERROR("put request failed: %s(%d) - aborting",
-                   mxm_error_string(mxm_req.base.error), mxm_req.base.error);
-        oshmem_shmem_abort(-1);
-        return OSHMEM_ERROR;
-    }
+    SPML_IKRIT_MXM_POST_SEND(mxm_req);
 
     wait.req = &mxm_req.base;
     wait.state = (mxm_req_state_t)(MXM_REQ_SENT | MXM_REQ_COMPLETED);
@@ -1342,7 +1349,7 @@ int mca_spml_ikrit_fence(void)
                  "Into fence with %d active puts on %d pes",
                  mca_spml_ikrit.n_active_puts, (int)opal_list_get_size(&mca_spml_ikrit.active_peers));
 
-    /* puts(unless are send sync) are completed by remote side lazily. That is either when remote decides to 
+    /* puts(unless are send sync) are completed by remote side lazily. That is either when remote decides to
      * ack window which can take hundreds of ms. So speed things up by doing fence */
     while (NULL != (item = opal_list_remove_first(&mca_spml_ikrit.active_peers))) {
         peer = (mxm_peer_t *) item;
@@ -1432,7 +1439,8 @@ int mca_spml_ikrit_send(void* buf,
     req.base.data.buffer.length = size == 0 ? sizeof(dummy_buf) : size;
     req.base.data.buffer.memh = NULL;
 
-    mxm_req_send(&req);
+    SPML_IKRIT_MXM_POST_SEND(req);
+
     mca_spml_irkit_req_wait(&req.base);
     if (req.base.error != MXM_OK) {
         return OSHMEM_ERROR;
