@@ -1,10 +1,13 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2013      Mellanox Technologies, Inc.
+ * Copyright (c) 2013-2015 Mellanox Technologies, Inc.
  *                         All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 #define _GNU_SOURCE
@@ -13,6 +16,7 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include "opal/util/opal_environ.h"
 #include "oshmem_config.h"
 #include "orte/util/show_help.h"
 #include "shmem.h"
@@ -23,7 +27,6 @@
 #include "oshmem/mca/spml/ikrit/spml_ikrit.h"
 
 #include "orte/util/show_help.h"
-#include "opal/util/opal_environ.h"
 
 static int mca_spml_ikrit_component_register(void);
 static int mca_spml_ikrit_component_open(void);
@@ -38,26 +41,23 @@ mca_spml_base_component_2_0_0_t mca_spml_ikrit_component = {
     /* First, the mca_base_component_t struct containing meta
        information about the component itself */
 
-    {
-      MCA_SPML_BASE_VERSION_2_0_0,
-    
-      "ikrit",                        /* MCA component name */
-      OSHMEM_MAJOR_VERSION,           /* MCA component major version */
-      OSHMEM_MINOR_VERSION,           /* MCA component minor version */
-      OSHMEM_RELEASE_VERSION,         /* MCA component release version */
-      mca_spml_ikrit_component_open,  /* component open */
-      mca_spml_ikrit_component_close, /* component close */
-      NULL,
-      mca_spml_ikrit_component_register
+    .spmlm_version = {
+        MCA_SPML_BASE_VERSION_2_0_0,
+
+        .mca_component_name = "ikrit",
+        MCA_BASE_MAKE_VERSION(component, OSHMEM_MAJOR_VERSION, OSHMEM_MINOR_VERSION,
+                              OSHMEM_RELEASE_VERSION),
+        .mca_open_component = mca_spml_ikrit_component_open,
+        .mca_close_component = mca_spml_ikrit_component_close,
+        .mca_register_component_params = mca_spml_ikrit_component_register,
     },
-    {
+    .spmlm_data = {
         /* The component is checkpoint ready */
         MCA_BASE_METADATA_PARAM_CHECKPOINT
     },
 
-    mca_spml_ikrit_component_init,    /* component init */
-    mca_spml_ikrit_component_fini     /* component finalize */
-    
+    .spmlm_init = mca_spml_ikrit_component_init,
+    .spmlm_finalize = mca_spml_ikrit_component_fini,
 };
 
 #if MXM_API >= MXM_VERSION(2,1)
@@ -93,21 +93,8 @@ static inline int set_mxm_tls()
 {
     char *tls;
 
-    /*
-     * Set DC defaults optimized for shmem
-     */
-    opal_setenv("MXM_OSHMEM_DC_QP_LIMIT",      "2", 0, &environ);
-    opal_setenv("MXM_OSHMEM_DC_RNDV_QP_LIMIT", "2", 0, &environ);
-    opal_setenv("MXM_OSHMEM_DC_MSS",          "8196", 0, &environ);
-
     /* disable dci pull for rdma ops. Use single pool.
-     * Pool size is controlled by MXM_DC_QP_LIMIT 
-     * variable
-     */
-    setenv("MXM_OSHMEM_DC_RNDV_QP_LIMIT", 0, 0);
-
-    /* disable dci pull for rdma ops. Use single pool.
-     * Pool size is controlled by MXM_DC_QP_LIMIT 
+     * Pool size is controlled by MXM_DC_QP_LIMIT
      * variable
      */
     setenv("MXM_OSHMEM_DC_RNDV_QP_LIMIT", "0", 0);
@@ -120,7 +107,7 @@ static inline int set_mxm_tls()
     tls = getenv("MXM_TLS");
     if (NULL == tls) {
         opal_setenv("MXM_OSHMEM_TLS", mca_spml_ikrit.mxm_tls, 1, &environ);
-        return OSHMEM_SUCCESS;
+        return check_mxm_tls("MXM_OSHMEM_TLS");
     }
     if (OSHMEM_SUCCESS == check_mxm_tls("MXM_TLS")) {
         opal_setenv("MXM_OSHMEM_TLS", tls, 1, &environ);
@@ -131,15 +118,18 @@ static inline int set_mxm_tls()
 
 static inline int check_mxm_hw_tls(char *v, char *tls)
 {
-    if ((0 == strcmp(tls, "rc") || 0 == strcmp(tls, "dc"))) {
-        return OSHMEM_SUCCESS;
-    }
+	if (v && tls) {
+        if ((0 == strcmp(tls, "rc") || 0 == strcmp(tls, "dc"))) {
+            mca_spml_ikrit.ud_only = 0;
+            return OSHMEM_SUCCESS;
+        }
 
-    if (strstr(tls, "ud") &&
+        if (strstr(tls, "ud") &&
             (NULL == strstr(tls, "rc") && NULL == strstr(tls, "dc") &&
              NULL == strstr(tls, "shm"))) {
-        return OSHMEM_SUCCESS;
-    }
+            return OSHMEM_SUCCESS;
+        }
+	}
 
     orte_show_help("help-oshmem-spml-ikrit.txt", "mxm tls", true,
                     v, tls);
@@ -153,8 +143,10 @@ static inline int set_mxm_hw_rdma_tls()
     }
     opal_setenv("MXM_OSHMEM_HW_RDMA_RC_QP_LIMIT", "-1", 0, &environ);
     opal_setenv("MXM_OSHMEM_HW_RDMA_TLS", "rc", 0, &environ);
+    SPML_VERBOSE(5, "Additional communication channel is enabled. Transports are: %s",
+                 getenv("MXM_OSHMEM_HW_RDMA_TLS"));
 
-    return check_mxm_hw_tls("MXM_OSHMEM_HW_RDMA_TLS", 
+    return check_mxm_hw_tls("MXM_OSHMEM_HW_RDMA_TLS",
             getenv("MXM_OSHMEM_HW_RDMA_TLS"));
 }
 
@@ -216,11 +208,10 @@ static int mca_spml_ikrit_component_register(void)
                                        "create separate reliable connection channel",
                                        &mca_spml_ikrit.hw_rdma_channel);
 
-    if (!mca_spml_ikrit.hw_rdma_channel) {
+    if (!mca_spml_ikrit.hw_rdma_channel)
         v = "ud,self";
-    } else {
+    else
         v = "rc,ud,self";
-    }
     mca_spml_ikrit_param_register_string("mxm_tls",
                                          v,
                                          "[string] TL channels for MXM",
@@ -307,6 +298,8 @@ static int mca_spml_ikrit_component_open(void)
     mca_spml_ikrit.ud_only = 1;
     mca_spml_ikrit.mxm_ctx_opts->ptl_bitmap = (MXM_BIT(MXM_PTL_SELF) | MXM_BIT(MXM_PTL_RDMA));
 #endif
+    SPML_VERBOSE(5, "UD only mode is %s",
+                 mca_spml_ikrit.ud_only ? "enabled" : "disabled");
 
     err = mxm_init(mca_spml_ikrit.mxm_ctx_opts, &mca_spml_ikrit.mxm_context);
     if (MXM_OK != err) {
@@ -349,9 +342,8 @@ static int mca_spml_ikrit_component_close(void)
 #else
         mxm_config_free_ep_opts(mca_spml_ikrit.mxm_ep_opts);
         mxm_config_free_context_opts(mca_spml_ikrit.mxm_ctx_opts);
-        if (mca_spml_ikrit.hw_rdma_channel) {
+        if (mca_spml_ikrit.hw_rdma_channel)
             mxm_config_free_ep_opts(mca_spml_ikrit.mxm_ep_hw_rdma_opts);
-        }
 #endif
     }
     mca_spml_ikrit.mxm_mq = NULL;
