@@ -23,15 +23,11 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif /* HAVE_SYS_TYPES_H */
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif /* HAVE_STRING_H */
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif /* HAVE_NETDB_H */
-#ifdef HAVE_TIME_H
 #include <time.h>
-#endif /* HAVE_NETDB_H */
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif /* HAVE_SYS_STAT_H */
@@ -193,7 +189,7 @@ segment_create(map_segment_t *ds_buf,
     /* Open device */
     if (NULL != mca_sshmem_verbs_component.hca_name) {
         for (i = 0; i < num_devs; i++) {
-            if (0 == strcmp(mca_sshmem_verbs_component.hca_name, ibv_get_device_name(device->ib_devs[0]))) {
+            if (0 == strcmp(mca_sshmem_verbs_component.hca_name, ibv_get_device_name(device->ib_devs[i]))) {
                 device->ib_dev = device->ib_devs[i];
                 break;
             }
@@ -245,21 +241,30 @@ segment_create(map_segment_t *ds_buf,
     if (!rc) {
         void *addr = NULL;
         struct ibv_mr *ib_mr = NULL;
-        int access_flag = IBV_ACCESS_LOCAL_WRITE |
+        uint64_t access_flag = IBV_ACCESS_LOCAL_WRITE |
                           IBV_ACCESS_REMOTE_WRITE |
                           IBV_ACCESS_REMOTE_READ;
-        int exp_access_flag = 0;
+        uint64_t exp_access_flag = 0;
 
         OBJ_CONSTRUCT(&device->ib_mr_array, opal_value_array_t);
         opal_value_array_init(&device->ib_mr_array, sizeof(struct ibv_mr *));
 
-#if defined(MPAGE_ENABLE) && (MPAGE_ENABLE > 0)
+#if (MPAGE_ENABLE > 0)
         exp_access_flag = IBV_EXP_ACCESS_ALLOCATE_MR |
                           IBV_EXP_ACCESS_SHARED_MR_USER_READ |
                           IBV_EXP_ACCESS_SHARED_MR_USER_WRITE;
 #endif /* MPAGE_ENABLE */
 
-        struct ibv_exp_reg_mr_in in = {device->ib_pd, addr, size, access_flag, exp_access_flag, 0};
+        struct ibv_exp_reg_mr_in in = {device->ib_pd, addr, size, access_flag|exp_access_flag, 0};
+
+#if MPAGE_HAVE_IBV_EXP_REG_MR_CREATE_FLAGS
+        if (0 == mca_sshmem_verbs_component.has_shared_mr) {
+            in.addr = (void *)mca_sshmem_base_start_address;
+            in.comp_mask    = IBV_EXP_REG_MR_CREATE_FLAGS;
+            in.create_flags = IBV_EXP_REG_MR_CREATE_CONTIG;
+            in.exp_access   = access_flag;
+        }
+#endif
         ib_mr = ibv_exp_reg_mr(&in);
         if (NULL == ib_mr) {
             OPAL_OUTPUT_VERBOSE(
@@ -273,21 +278,24 @@ segment_create(map_segment_t *ds_buf,
             opal_value_array_append_item(&device->ib_mr_array, &ib_mr);
         }
 
-#if defined(MPAGE_ENABLE) && (MPAGE_ENABLE > 0)
-        if (!rc) {
+#if (MPAGE_ENABLE > 0)
+        if (!rc && mca_sshmem_verbs_component.has_shared_mr) {
             access_flag = IBV_ACCESS_LOCAL_WRITE |
                           IBV_ACCESS_REMOTE_WRITE |
                           IBV_ACCESS_REMOTE_READ|
                           IBV_EXP_ACCESS_NO_RDMA;
 
             addr = (void *)mca_sshmem_base_start_address;
-            struct ibv_exp_reg_shared_mr_in in = {0,device->ib_mr_shared->handle,device->ib_pd, addr, access_flag};
+            struct ibv_exp_reg_shared_mr_in in;
+            mca_sshmem_verbs_fill_shared_mr(&in, device->ib_pd, device->ib_mr_shared->handle, addr, access_flag);
             ib_mr = ibv_exp_reg_shared_mr(&in);
             if (NULL == ib_mr) {
                 OPAL_OUTPUT_VERBOSE(
                     (5, oshmem_sshmem_base_framework.framework_output,
-                        "error to ibv_reg_shared_mr() %llu bytes errno says %d: %s",
-                        (unsigned long long)size, errno, strerror(errno))
+                        "error to ibv_reg_shared_mr() %llu bytes errno says %d: %s has_shared_mr: %d",
+                        (unsigned long long)size, errno, strerror(errno),
+                        mca_sshmem_verbs_component.has_shared_mr
+                        )
                     );
                 rc = OSHMEM_ERR_OUT_OF_RESOURCE;
             } else {
@@ -297,16 +305,21 @@ segment_create(map_segment_t *ds_buf,
 #endif /* MPAGE_ENABLE */
 
         if (!rc) {
-            assert(size == device->ib_mr_shared->length);
-
             OPAL_OUTPUT_VERBOSE(
                 (70, oshmem_sshmem_base_framework.framework_output,
-                "ibv device %s",
-                ibv_get_device_name(device->ib_dev))
+                "ibv device %s shared_mr: %d",
+                ibv_get_device_name(device->ib_dev),
+                mca_sshmem_verbs_component.has_shared_mr)
                 );
 
-            ds_buf->type = MAP_SEGMENT_ALLOC_IBV;
-            ds_buf->seg_id = device->ib_mr_shared->handle;
+            if (mca_sshmem_verbs_component.has_shared_mr) {
+                assert(size == device->ib_mr_shared->length);
+                ds_buf->type = MAP_SEGMENT_ALLOC_IBV;
+                ds_buf->seg_id = device->ib_mr_shared->handle;
+            } else {
+                ds_buf->type = MAP_SEGMENT_ALLOC_IBV_NOSHMR;
+                ds_buf->seg_id = MAP_SEGMENT_SHM_INVALID;
+            }
             ds_buf->seg_base_addr = ib_mr->addr;
             ds_buf->seg_size = size;
             ds_buf->end = (void*)((uintptr_t)ds_buf->seg_base_addr + ds_buf->seg_size);
@@ -350,13 +363,13 @@ segment_attach(map_segment_t *ds_buf, sshmem_mkey_t *mkey)
         mca_sshmem_verbs_component.mr_interleave_factor * 1024ULL * 1024ULL * 1024ULL * mr_count);
     {
         struct ibv_mr *ib_mr = NULL;
-        int access_flag = IBV_ACCESS_LOCAL_WRITE |
+        uint64_t access_flag = IBV_ACCESS_LOCAL_WRITE |
             IBV_ACCESS_REMOTE_WRITE |
             IBV_ACCESS_REMOTE_READ |
             IBV_EXP_ACCESS_NO_RDMA;
-        struct ibv_exp_reg_shared_mr_in in = {
-            0, mkey->u.key, device->ib_pd, addr, access_flag};
+        struct ibv_exp_reg_shared_mr_in in;
 
+        mca_sshmem_verbs_fill_shared_mr(&in, device->ib_pd, mkey->u.key, addr, access_flag);
         ib_mr = ibv_exp_reg_shared_mr(&in);
         if (NULL == ib_mr) {
             mkey->va_base = (void *)-1;
@@ -399,6 +412,7 @@ segment_detach(map_segment_t *ds_buf, sshmem_mkey_t *mkey)
 {
     int rc = OSHMEM_SUCCESS;
     openib_device_t *device = &memheap_device;
+    int i;
 
     assert(ds_buf);
 
@@ -412,12 +426,12 @@ segment_detach(map_segment_t *ds_buf, sshmem_mkey_t *mkey)
     );
 
     if (device) {
-        if (opal_value_array_get_size(&device->ib_mr_array)) {
+        if (0 < (i = opal_value_array_get_size(&device->ib_mr_array))) {
             struct ibv_mr** array;
             struct ibv_mr* ib_mr = NULL;
             array = OPAL_VALUE_ARRAY_GET_BASE(&device->ib_mr_array, struct ibv_mr *);
-            while (opal_value_array_get_size(&device->ib_mr_array) > 0) {
-                ib_mr = array[0];
+            for (i--;i >= 0; i--) {
+                ib_mr = array[i];
                 if(ibv_dereg_mr(ib_mr)) {
                     OPAL_OUTPUT_VERBOSE(
                         (5, oshmem_sshmem_base_framework.framework_output,
@@ -426,7 +440,7 @@ segment_detach(map_segment_t *ds_buf, sshmem_mkey_t *mkey)
                         );
                     rc = OSHMEM_ERROR;
                 }
-                opal_value_array_remove_item(&device->ib_mr_array, 0);
+                opal_value_array_remove_item(&device->ib_mr_array, i);
             }
 
             if (!rc && device->ib_mr_shared) {
