@@ -2,9 +2,9 @@
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 #include "oshmem_config.h"
@@ -17,7 +17,7 @@
 #include "oshmem/mca/spml/spml.h"
 
 /*
- * The strided get routines copy strided data located on a remote PE to a local strided data object. 
+ * The strided get routines copy strided data located on a remote PE to a local strided data object.
  * The strided get routines retrieve array data available at address source on remote PE (pe).
  * The elements of the source array are separated by a stride sst. Once the data is received,
  * it is stored at the local memory address target, separated by stride tst. The routines return
@@ -61,9 +61,6 @@
 #pragma weak shmem_iget32 = pshmem_iget32
 #pragma weak shmem_iget64 = pshmem_iget64
 #pragma weak shmem_iget128 = pshmem_iget128
-
-/* Deprecated */
-#pragma weak shmem_iget16 = pshmem_iget16
 #include "oshmem/shmem/c/profile/defines.h"
 #endif
 
@@ -102,6 +99,3 @@ SHMEM_TYPE_IGETMEM(_iget16, 2, shmemx)
 SHMEM_TYPE_IGETMEM(_iget32, 4, shmem)
 SHMEM_TYPE_IGETMEM(_iget64, 8, shmem)
 SHMEM_TYPE_IGETMEM(_iget128, 16, shmem)
-
-/* Deprecated */
-SHMEM_TYPE_IGETMEM(_iget16, 2, shmem)
