@@ -3,9 +3,9 @@
  *                         All rights reserved.
  * Copyright (c) 2013 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -16,19 +16,25 @@
 #include "oshmem/runtime/runtime.h"
 #include "stdio.h"
 
+#if OSHMEM_PROFILING
+#include "oshmem/shmem/fortran/profile/pbindings.h"
+SHMEM_GENERATE_WEAK_BINDINGS(SHMEM_PUT, shmem_put)
+#include "oshmem/shmem/fortran/profile/defines.h"
+#endif
+
 SHMEM_GENERATE_FORTRAN_BINDINGS_SUB (void,
         SHMEM_PUT,
         shmem_put_,
         shmem_put__,
         shmem_put_f,
-        (FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI_Fint *length, MPI_Fint *pe), 
+        (FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI_Fint *length, MPI_Fint *pe),
         (target,source,length,pe) )
 
 void shmem_put_f(FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI_Fint *length, MPI_Fint *pe)
 {
-    shmem_put(FPTR_2_VOID_PTR(target), 
-        FPTR_2_VOID_PTR(source), 
-        OMPI_FINT_2_INT(*length), 
+    shmem_put(FPTR_2_VOID_PTR(target),
+        FPTR_2_VOID_PTR(source),
+        OMPI_FINT_2_INT(*length),
         OMPI_FINT_2_INT(*pe));
 }
 
